from migen import *
from migen.fhdl import verilog


class SimCase:
    def setUp(self, *args, **kwargs):
        self.tb = self.TestBench(*args, **kwargs)

    def test_to_verilog(self):
        verilog.convert(self.tb)

    def run_with(self, generator):
        run_simulation(self.tb, generator)



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform, XC3SProg, VivadoProgrammer, iMPACT
from migen.build.xilinx.ise import XilinxISEToolchain


_io = [
    ("user_led", 0, Pins("AB8"), IOStandard("LVCMOS15")),
    ("user_led", 1, Pins("AA8"), IOStandard("LVCMOS15")),
    ("user_led", 2, Pins("AC9"), IOStandard("LVCMOS15")),
    ("user_led", 3, Pins("AB9"), IOStandard("LVCMOS15")),
    ("user_led", 4, Pins("AE26"), IOStandard("LVCMOS25")),
    ("user_led", 5, Pins("G19"), IOStandard("LVCMOS25")),
    ("user_led", 6, Pins("E18"), IOStandard("LVCMOS25")),
    ("user_led", 7, Pins("F16"), IOStandard("LVCMOS25")),

    ("cpu_reset", 0, Pins("AB7"), IOStandard("LVCMOS15")),

    ("user_btn_c", 0, Pins("G12"), IOStandard("LVCMOS25")),
    ("user_btn_n", 0, Pins("AA12"), IOStandard("LVCMOS15")),
    ("user_btn_s", 0, Pins("AB12"), IOStandard("LVCMOS15")),
    ("user_btn_w", 0, Pins("AC6"), IOStandard("LVCMOS15")),
    ("user_btn_e", 0, Pins("AG5"), IOStandard("LVCMOS15")),

    ("user_dip_btn", 0, Pins("Y29"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 1, Pins("W29"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 2, Pins("AA28"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 3, Pins("Y28"), IOStandard("LVCMOS25")),

    ("user_sma_clock", 0,
        Subsignal("p", Pins("L25"), IOStandard("LVDS_25"),
            Misc("DIFF_TERM=TRUE")),
        Subsignal("n", Pins("K25"), IOStandard("LVDS_25"),
            Misc("DIFF_TERM=TRUE"))
    ),
    ("user_sma_clock_p", 0, Pins("L25"), IOStandard("LVCMOS25")),
    ("user_sma_clock_n", 0, Pins("K25"), IOStandard("LVCMOS25")),

    ("user_sma_gpio_p", 0, Pins("Y23"), IOStandard("LVCMOS25")),
    ("user_sma_gpio_n", 0, Pins("Y24"), IOStandard("LVCMOS25")),

    ("clk200", 0,
        Subsignal("p", Pins("AD12"), IOStandard("LVDS")),
        Subsignal("n", Pins("AD11"), IOStandard("LVDS"))
    ),

    ("clk156", 0,
        Subsignal("p", Pins("K28"), IOStandard("LVDS_25")),
        Subsignal("n", Pins("K29"), IOStandard("LVDS_25"))
    ),

    ("i2c", 0,
        Subsignal("scl", Pins("K21")),
        Subsignal("sda", Pins("L21")),
        IOStandard("LVCMOS25")),

    ("serial", 0,
        Subsignal("cts", Pins("L27")),
        Subsignal("rts", Pins("K23")),
        Subsignal("tx", Pins("K24")),
        Subsignal("rx", Pins("M19")),
        IOStandard("LVCMOS25")),

    ("spiflash", 0,  # clock needs to be accessed through STARTUPE2
        Subsignal("cs_n", Pins("U19")),
        Subsignal("dq", Pins("P24", "R25", "R20", "R21")),
        IOStandard("LVCMOS25")
    ),

    ("mmc", 0,
        Subsignal("wp", Pins("Y21")),
        Subsignal("det", Pins("AA21")),
        Subsignal("cmd", Pins("AB22")),
        Subsignal("clk", Pins("AB23")),
        Subsignal("dat", Pins("AC20 AA23 AA22 AC21")),
        IOStandard("LVCMOS25")),

    ("mmc_spi", 0,
        Subsignal("miso", Pins("AC20"), Misc("PULLUP")),
        Subsignal("clk", Pins("AB23")),
        Subsignal("mosi", Pins("AB22")),
        Subsignal("cs_n", Pins("AC21")),
        IOStandard("LVCMOS25")),

    ("lcd", 0,
        Subsignal("db", Pins("AA13 AA10 AA11 Y10")),
        Subsignal("e", Pins("AB10")),
        Subsignal("rs", Pins("Y11")),
        Subsignal("rw", Pins("AB13")),
        IOStandard("LVCMOS15")),

    ("rotary", 0,
        Subsignal("a", Pins("Y26")),
        Subsignal("b", Pins("Y25")),
        Subsignal("push", Pins("AA26")),
        IOStandard("LVCMOS25")),

    ("hdmi", 0,
        Subsignal("d", Pins("B23 A23 E23 D23 F25 E25 E24 D24 F26 E26 G23 G24 J19 H19 L17 L18 K19 K20")),
        Subsignal("de", Pins("H17")),
        Subsignal("clk", Pins("K18")),
        Subsignal("vsync", Pins("H20")),
        Subsignal("hsync", Pins("J18")),
        Subsignal("int", Pins("AH24")),
        Subsignal("spdif", Pins("J17")),
        Subsignal("spdif_out", Pins("G20")),
        IOStandard("LVCMOS25")),

    ("ddram", 0,
        Subsignal("a", Pins(
            "AH12 AG13 AG12 AF12 AJ12 AJ13 AJ14 AH14",
            "AK13 AK14 AF13 AE13 AJ11 AH11 AK10 AK11"),
            IOStandard("SSTL15")),
        Subsignal("ba", Pins("AH9 AG9 AK9"), IOStandard("SSTL15")),
        Subsignal("ras_n", Pins("AD9"), IOStandard("SSTL15")),
        Subsignal("cas_n", Pins("AC11"), IOStandard("SSTL15")),
        Subsignal("we_n", Pins("AE9"), IOStandard("SSTL15")),
        Subsignal("cs_n", Pins("AC12"), IOStandard("SSTL15")),
        Subsignal("dm", Pins("Y16 AB17 AF17 AE16 AK5 AJ3 AF6 AC7"),
            IOStandard("SSTL15")),
        Subsignal("dq", Pins(
            "AA15 AA16 AC14 AD14 AA17 AB15 AE15 Y15",
            "AB19 AD16 AC19 AD17 AA18 AB18 AE18 AD18",
            "AG19 AK19 AG18 AF18 AH19 AJ19 AE19 AD19",
            "AK16 AJ17 AG15 AF15 AH17 AG14 AH15 AK15",
            "AK8 AK6 AG7 AF7 AF8 AK4 AJ8 AJ6",
            "AH5 AH6 AJ2 AH2 AH4 AJ4 AK1 AJ1",
            "AF1 AF2 AE4 AE3 AF3 AF5 AE1 AE5",
            "AC1 AD3 AC4 AC5 AE6 AD6 AC2 AD4"),
            IOStandard("SSTL15_T_DCI")),
        Subsignal("dqs_p", Pins("AC16 Y19 AJ18 AH16 AH7 AG2 AG4 AD2"),
            IOStandard("DIFF_SSTL15")),
        Subsignal("dqs_n", Pins("AC15 Y18 AK18 AJ16 AJ7 AH1 AG3 AD1"),
            IOStandard("DIFF_SSTL15")),
        Subsignal("clk_p", Pins("AG10"), IOStandard("DIFF_SSTL15")),
        Subsignal("clk_n", Pins("AH10"), IOStandard("DIFF_SSTL15")),
        Subsignal("cke", Pins("AF10"), IOStandard("SSTL15")),
        Subsignal("odt", Pins("AD8"), IOStandard("SSTL15")),
        Subsignal("reset_n", Pins("AK3"), IOStandard("LVCMOS15")),
        Misc("SLEW=FAST"),
        Misc("VCCAUX_IO=HIGH")
    ),

    ("eth_clocks", 0,
        Subsignal("tx", Pins("M28")),
        Subsignal("gtx", Pins("K30")),
        Subsignal("rx", Pins("U27")),
        IOStandard("LVCMOS25")
    ),
    ("eth", 0,
        Subsignal("rst_n", Pins("L20")),
        Subsignal("int_n", Pins("N30")),
        Subsignal("mdio", Pins("J21")),
        Subsignal("mdc", Pins("R23")),
        Subsignal("rx_dv", Pins("R28")),
        Subsignal("rx_er", Pins("V26")),
        Subsignal("rx_data", Pins("U30 U25 T25 U28 R19 T27 T26 T28")),
        Subsignal("tx_en", Pins("M27")),
        Subsignal("tx_er", Pins("N29")),
        Subsignal("tx_data", Pins("N27 N25 M29 L28 J26 K26 L30 J28")),
        Subsignal("col", Pins("W19")),
        Subsignal("crs", Pins("R30")),
        IOStandard("LVCMOS25")
    ),

    ("pcie_x1", 0,
        Subsignal("rst_n", Pins("G25"), IOStandard("LVCMOS25")),
        Subsignal("clk_p", Pins("U8")),
        Subsignal("clk_n", Pins("U7")),
        Subsignal("rx_p", Pins("M6")),
        Subsignal("rx_n", Pins("M5")),
        Subsignal("tx_p", Pins("L4")),
        Subsignal("tx_n", Pins("L3"))
    ),
    ("pcie_x2", 0,
        Subsignal("rst_n", Pins("G25"), IOStandard("LVCMOS25")),
        Subsignal("clk_p", Pins("U8")),
        Subsignal("clk_n", Pins("U7")),
        Subsignal("rx_p", Pins("M6 P6")),
        Subsignal("rx_n", Pins("M5 P5")),
        Subsignal("tx_p", Pins("L4 M2")),
        Subsignal("tx_n", Pins("L3 M1"))
    ),
    ("pcie_x4", 0,
        Subsignal("rst_n", Pins("G25"), IOStandard("LVCMOS25")),
        Subsignal("clk_p", Pins("U8")),
        Subsignal("clk_n", Pins("U7")),
        Subsignal("rx_p", Pins("M6 P6 R4 T6")),
        Subsignal("rx_n", Pins("M5 P5 R3 T5")),
        Subsignal("tx_p", Pins("L4 M2 N4 P2")),
        Subsignal("tx_n", Pins("L3 M1 N3 P1"))
    ),
    ("pcie_x8", 0,
        Subsignal("rst_n", Pins("G25"), IOStandard("LVCMOS25")),
        Subsignal("clk_p", Pins("U8")),
        Subsignal("clk_n", Pins("U7")),
        Subsignal("rx_p", Pins("M6 P6 R4 T6 V6 W4 Y6 AA4")),
        Subsignal("rx_n", Pins("M5 P5 R3 T5 V5 W3 Y5 AA3")),
        Subsignal("tx_p", Pins("L4 M2 N4 P2 T2 U4 V2 Y2")),
        Subsignal("tx_n", Pins("L3 M1 N3 P1 T1 U3 V1 Y1"))
    ),

    ("vadj_on_b", 0, Pins("J27"), IOStandard("LVCMOS25")),

    ("sgmii_clock", 0,
        Subsignal("p", Pins("G8")),
        Subsignal("n", Pins("G7"))
    ),
    ("user_sma_mgt_tx", 0,
        Subsignal("p", Pins("K2")),
        Subsignal("n", Pins("K1"))
    ),
    ("user_sma_mgt_rx", 0,
        Subsignal("p", Pins("K6")),
        Subsignal("n", Pins("K5"))
    ),
    ("sfp_tx", 0,  # inverted prior to HW rev 1.1
        Subsignal("p", Pins("H2")),
        Subsignal("n", Pins("H1"))
    ),
    ("sfp_rx", 0,  # inverted prior to HW rev 1.1
        Subsignal("p", Pins("G4")),
        Subsignal("n", Pins("G3"))
    ),
    ("sfp_tx_disable_n", 0, Pins("Y20"), IOStandard("LVCMOS25")),
    ("sfp_rx_los", 0, Pins("P19"), IOStandard("LVCMOS25")),

    ("si5324", 0,
        Subsignal("rst_n", Pins("AE20"), IOStandard("LVCMOS25")),
        Subsignal("int", Pins("AG24"), IOStandard("LVCMOS25"))
    ),
    ("si5324_clkin", 0,
        Subsignal("p", Pins("W27"), IOStandard("LVDS_25")),
        Subsignal("n", Pins("W28"), IOStandard("LVDS_25"))
    ),
    ("si5324_clkout", 0,
        Subsignal("p", Pins("L8")),
        Subsignal("n", Pins("L7"))
    ),
]


_connectors = [
    ("HPC", {
        "DP1_M2C_P": "D6",
        "DP1_M2C_N": "D5",
        "DP2_M2C_P": "B6",
        "DP2_M2C_N": "B5",
        "DP3_M2C_P": "A8",
        "DP3_M2C_N": "A7",
        "DP1_C2M_P": "C4",
        "DP1_C2M_N": "C3",
        "DP2_C2M_P": "B2",
        "DP2_C2M_N": "B1",
        "DP3_C2M_P": "A4",
        "DP3_C2M_N": "A3",
        "DP0_C2M_P": "D2",
        "DP0_C2M_N": "D1",
        "DP0_M2C_P": "E4",
        "DP0_M2C_N": "E3",
        "LA06_P": "H30",
        "LA06_N": "G30",
        "LA10_P": "D29",
        "LA10_N": "C30",
        "LA14_P": "B28",
        "LA14_N": "A28",
        "LA18_CC_P": "F21",
        "LA18_CC_N": "E21",
        "LA27_P": "C19",
        "LA27_N": "B19",
        "HA01_CC_P": "H14",
        "HA01_CC_N": "G14",
        "HA05_P": "F15",
        "HA05_N": "E16",
        "HA09_P": "F12",
        "HA09_N": "E13",
        "HA13_P": "L16",
        "HA13_N": "K16",
        "HA16_P": "L15",
        "HA16_N": "K15",
        "HA20_P": "K13",
        "HA20_N": "J13",
        "CLK1_M2C_P": "D17",
        "CLK1_M2C_N": "D18",
        "LA00_CC_P": "C25",
        "LA00_CC_N": "B25",
        "LA03_P": "H26",
        "LA03_N": "H27",
        "LA08_P": "E29",
        "LA08_N": "E30",
        "LA12_P": "C29",
        "LA12_N": "B29",
        "LA16_P": "B27",
        "LA16_N": "A27",
        "LA20_P": "E19",
        "LA20_N": "D19",
        "LA22_P": "C20",
        "LA22_N": "B20",
        "LA25_P": "G17",
        "LA25_N": "F17",
        "LA29_P": "C17",
        "LA29_N": "B17",
        "LA31_P": "G22",
        "LA31_N": "F22",
        "LA33_P": "H21",
        "LA33_N": "H22",
        "HA03_P": "C12",
        "HA03_N": "B12",
        "HA07_P": "B14",
        "HA07_N": "A15",
        "HA11_P": "B13",
        "HA11_N": "A13",
        "HA14_P": "J16",
        "HA14_N": "H16",
        "HA18_P": "K14",
        "HA18_N": "J14",
        "HA22_P": "L11",
        "HA22_N": "K11",
        "GBTCLK1_M2C_P": "E8",
        "GBTCLK1_M2C_N": "E7",
        "GBTCLK0_M2C_P": "C8",
        "GBTCLK0_M2C_N": "C7",
        "LA01_CC_P": "D26",
        "LA01_CC_N": "C26",
        "LA05_P": "G29",
        "LA05_N": "F30",
        "LA09_P": "B30",
        "LA09_N": "A30",
        "LA13_P": "A25",
        "LA13_N": "A26",
        "LA17_CC_P": "F20",
        "LA17_CC_N": "E20",
        "LA23_P": "B22",
        "LA23_N": "A22",
        "LA26_P": "B18",
        "LA26_N": "A18",
        "PG_M2C": "J29",
        "HA00_CC_P": "D12",
        "HA00_CC_N": "D13",
        "HA04_P": "F11",
        "HA04_N": "E11",
        "HA08_P": "E14",
        "HA08_N": "E15",
        "HA12_P": "C15",
        "HA12_N": "B15",
        "HA15_P": "H15",
        "HA15_N": "G15",
        "HA19_P": "H11",
        "HA19_N": "H12",
        "PRSNT_M2C_B": "M20",
        "CLK0_M2C_P": "D27",
        "CLK0_M2C_N": "C27",
        "LA02_P": "H24",
        "LA02_N": "H25",
        "LA04_P": "G28",
        "LA04_N": "F28",
        "LA07_P": "E28",
        "LA07_N": "D28",
        "LA11_P": "G27",
        "LA11_N": "F27",
        "LA15_P": "C24",
        "LA15_N": "B24",
        "LA19_P": "G18",
        "LA19_N": "F18",
        "LA21_P": "A20",
        "LA21_N": "A21",
        "LA24_P": "A16",
        "LA24_N": "A17",
        "LA28_P": "D16",
        "LA28_N": "C16",
        "LA30_P": "D22",
        "LA30_N": "C22",
        "LA32_P": "D21",
        "LA32_N": "C21",
        "HA02_P": "D11",
        "HA02_N": "C11",
        "HA06_P": "D14",
        "HA06_N": "C14",
        "HA10_P": "A11",
        "HA10_N": "A12",
        "HA17_CC_P": "G13",
        "HA17_CC_N": "F13",
        "HA21_P": "J11",
        "HA21_N": "J12",
        "HA23_P": "L12",
        "HA23_N": "L13",
        }
    ),
    ("LPC", {
        "GBTCLK0_M2C_P": "N8",
        "GBTCLK0_M2C_N": "N7",
        "LA01_CC_P": "AE23",
        "LA01_CC_N": "AF23",
        "LA05_P": "AG22",
        "LA05_N": "AH22",
        "LA09_P": "AK23",
        "LA09_N": "AK24",
        "LA13_P": "AB24",
        "LA13_N": "AC25",
        "LA17_CC_P": "AB27",
        "LA17_CC_N": "AC27",
        "LA23_P": "AH26",
        "LA23_N": "AH27",
        "LA26_P": "AK29",
        "LA26_N": "AK30",
        "CLK0_M2C_P": "AF22",
        "CLK0_M2C_N": "AG23",
        "LA02_P": "AF20",
        "LA02_N": "AF21",
        "LA04_P": "AH21",
        "LA04_N": "AJ21",
        "LA07_P": "AG25",
        "LA07_N": "AH25",
        "LA11_P": "AE25",
        "LA11_N": "AF25",
        "LA15_P": "AC24",
        "LA15_N": "AD24",
        "LA19_P": "AJ26",
        "LA19_N": "AK26",
        "LA21_P": "AG27",
        "LA21_N": "AG28",
        "LA24_P": "AG30",
        "LA24_N": "AH30",
        "LA28_P": "AE30",
        "LA28_N": "AF30",
        "LA30_P": "AB29",
        "LA30_N": "AB30",
        "LA32_P": "Y30",
        "LA32_N": "AA30",
        "LA06_P": "AK20",
        "LA06_N": "AK21",
        "LA10_P": "AJ24",
        "LA10_N": "AK25",
        "LA14_P": "AD21",
        "LA14_N": "AE21",
        "LA18_CC_P": "AD27",
        "LA18_CC_N": "AD28",
        "LA27_P": "AJ28",
        "LA27_N": "AJ29",
        "CLK1_M2C_P": "AG29",
        "CLK1_M2C_N": "AH29",
        "LA00_CC_P": "AD23",
        "LA00_CC_N": "AE24",
        "LA03_P": "AG20",
        "LA03_N": "AH20",
        "LA08_P": "AJ22",
        "LA08_N": "AJ23",
        "LA12_P": "AA20",
        "LA12_N": "AB20",
        "LA16_P": "AC22",
        "LA16_N": "AD22",
        "LA20_P": "AF26",
        "LA20_N": "AF27",
        "LA22_P": "AJ27",
        "LA22_N": "AK28",
        "LA25_P": "AC26",
        "LA25_N": "AD26",
        "LA29_P": "AE28",
        "LA29_N": "AF28",
        "LA31_P": "AD29",
        "LA31_N": "AE29",
        "LA33_P": "AC29",
        "LA33_N": "AC30",
        }
    ),
    ("XADC", {
        "GPIO0": "AB25",
        "GPIO1": "AA25",
        "GPIO2": "AB28",
        "GPIO3": "AA27",
        "VAUX0_N": "J24",
        "VAUX0_P": "J23",
        "VAUX8_N": "L23",
        "VAUX8_P": "L22",
        }
    ),
]


class Platform(XilinxPlatform):
    default_clk_name = "clk156"
    default_clk_period = 6.4

    def __init__(self, toolchain="vivado", programmer="xc3sprog"):
        XilinxPlatform.__init__(self, "xc7k325t-ffg900-2", _io, _connectors,
            toolchain=toolchain)
        if toolchain == "ise":
            self.toolchain.bitgen_opt = "-g LCK_cycle:6 -g Binary:Yes -w -g ConfigRate:12 -g SPI_buswidth:4"
        elif toolchain == "vivado":
            self.toolchain.bitstream_commands = ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"]
            self.toolchain.additional_commands = ["write_cfgmem -force -format bin -interface spix4 -size 16 -loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]
        self.programmer = programmer

    def create_programmer(self):
        if self.programmer == "xc3sprog":
            return XC3SProg("jtaghs1_fast", "bscan_spi_kc705.bit")
        elif self.programmer == "vivado":
            return VivadoProgrammer()
        elif self.programmer == "impact":
            return iMPACT()
        else:
            raise ValueError("{} programmer is not supported".format(programmer))

    def do_finalize(self, fragment):
        XilinxPlatform.do_finalize(self, fragment)
        try:
            self.add_period_constraint(self.lookup_request("clk200").p, 5.0)
        except ConstraintError:
            pass
        try:
            self.add_period_constraint(self.lookup_request("eth_clocks").rx, 8.0)
        except ConstraintError:
            pass
        try:
            self.add_period_constraint(self.lookup_request("eth_clocks").tx, 8.0)
        except ConstraintError:
            pass
        if isinstance(self.toolchain, XilinxISEToolchain):
            self.add_platform_command("CONFIG DCI_CASCADE = \"33 32 34\";")
        else:
            self.add_platform_command("set_property DCI_CASCADE {{32 34}} [get_iobanks 33]")



from itertools import count
import tempfile
import os
from collections import OrderedDict
import shutil

from migen.fhdl.namer import build_namespace


def vcd_codes():
    codechars = [chr(i) for i in range(33, 127)]
    for n in count():
        q, r = divmod(n, len(codechars))
        code = codechars[r]
        while q > 0:
            q, r = divmod(q, len(codechars))
            code = codechars[r] + code
        yield code


class VCDWriter:
    def __init__(self, filename):
        self.filename = filename
        self.buffer_file = tempfile.TemporaryFile(
            dir=os.path.dirname(filename), mode="w+")
        self.codegen = vcd_codes()
        self.codes = OrderedDict()
        self.signal_values = dict()
        self.t = 0

    def _get_code(self, signal):
        try:
            code = self.codes[signal]
        except KeyError:
            code = next(self.codegen)
            self.codes[signal] = code
        return code

    def _write_primitive_value(self, f, signal, value):
        l = len(signal)
        if value < 0:
            value += 2**l
        if l > 1:
            fmtstr = "b{:0" + str(l) + "b} {}\n"
        else:
            fmtstr = "{}{}\n"
        code = self._get_code(signal)
        f.write(fmtstr.format(value, code))

    def _write_enum_value(self, f, signal, value):
        val = "b"
        for c in signal._enumeration[value].encode():
            val += "{:08b}".format(c)
        code = self._get_code(signal)
        f.write("{} {}\n".format(val, code))

    def _write_value(self, f, signal, value):
        if hasattr(signal, "_enumeration"):
            self._write_enum_value(self.buffer_file, signal, value)
        else:
            self._write_primitive_value(self.buffer_file, signal, value)

    def set(self, signal, value):
        if signal not in self.signal_values or self.signal_values[signal] != value:
            self._write_value(self.buffer_file, signal, value)
            self.signal_values[signal] = value

    def delay(self, delay):
        self.t += delay
        self.buffer_file.write("#{}\n".format(self.t))

    def close(self):
        out = open(self.filename, "w")
        try:
            ns = build_namespace(self.codes.keys())
            for signal, code in self.codes.items():
                name = ns.get_name(signal)
                if hasattr(signal, "_enumeration"):
                    size = max([len(v) for v in signal._enumeration.values()])*8
                else:
                    size = len(signal)
                out.write("$var wire {size} {code} {name} $end\n"
                          .format(name=name, code=code, size=size))
            out.write("$dumpvars\n")
            for signal in self.codes.keys():
                self._write_value(out, signal, signal.reset.value)
            out.write("$end\n")
            out.write("#0\n")

            self.buffer_file.seek(0)
            shutil.copyfileobj(self.buffer_file, out)
            self.buffer_file.close()
        finally:
            out.close()


class DummyVCDWriter:
    def set(self, signal, value):
        pass

    def delay(self, delay):
        pass

    def close(self):
        pass



import bisect

from migen import *
from migen.build.generic_platform import *
from migen.build.lattice import LatticePlatform

_connectors = [
    # Pins 1-22
    ("GPIO", "13 14 16 17 20 21 23 25 26 27 28 4 5 8 9 10 11 12"),
]


# Default peripherals.
# Only on AX.
led = [
    ("user_led", 0, Pins("GPIO:0"), IOStandard("LVCMOS33"))
]

serial = [
    ("serial", 0,
        Subsignal("tx", Pins("GPIO:2")),
        Subsignal("rx", Pins("GPIO:3")),
        IOStandard("LVCMOS33")
     )
]


class MachClock(Module):
    # Diamond will complain that the frequency of the oscillator doesn't match
    # if it doesn't match one of the below frequencies. Since we tend to deal
    # with periods in nanoseconds, choose the closest frequency (in MHz)
    # within +/-5%. These numbers were extracted from
    # "MachXO2 sysCLOCK PLL Design and Usage Guide"
    supported_freqs = [
        2.08, 2.15, 2.22, 2.29, 2.38, 2.46, 2.56, 2.66, 2.77, 2.89,
        3.02, 3.17, 3.33, 3.50, 3.69, 3.91, 4.16, 4.29, 4.43, 4.59,
        4.75, 4.93, 5.12, 5.32, 5.54, 5.78, 6.05, 6.33, 6.65, 7.00,
        7.39, 7.82, 8.31, 8.58, 8.87, 9.17, 9.50, 9.85, 10.23, 10.64,
        11.08, 11.57, 12.09, 12.67, 13.30, 14.00, 14.78, 15.65, 15.65, 16.63,
        17.73, 19.00, 20.46, 22.17, 24.18, 26.60, 29.56, 33.25, 38.00, 44.33,
        53.20, 66.50, 88.67, 133.00
    ]

    def __init__(self, period, out):
        freq_str = "{:.2f}".format(self.nearest_freq(period))
        self.specials += Instance("OSCH",
                                  p_NOM_FREQ=freq_str,
                                  i_STDBY=C(0),
                                  o_OSC=out
                                  )

    def nearest_freq(self, period):
        target_freq = 1000.0/period

        # A None match is defined to not be in range. Handles special cases
        # described below.
        def in_range(actual, err=0.05):
            return abs((target_freq - actual)/target_freq) <= err if actual else False

        # Agreed-upon algorithm:
        # https://github.com/m-labs/migen/pull/111#discussion_r189687411
        i = bisect.bisect_right(self.supported_freqs, target_freq)

        # Special case: desired freq was < 2.08, but could still within +5%
        lo_match = self.supported_freqs[i-1] if i else None

        # Special case: desired freq was > 133.00, but could still within -5%
        hi_match = self.supported_freqs[i] if i < len(self.supported_freqs) else None

        if in_range(lo_match):
            selected_freq = lo_match
        elif in_range(hi_match):
            selected_freq = hi_match
        else:
            raise ValueError("Clock period out of range (-/+5%) for internal oscillator.")

        return selected_freq


class OschRouting(Module):
    def __init__(self):
        self.osch_used = False  # Only one default clock,
        # even if > 1 OSCH is possible, so run only once.
        self.mach_clk_sig = Signal()

    def mk_clk(self, name, clk_period):
        if not self.osch_used:
            self.mach_clk_sig.name_override = name
            self.submodules.mclk = MachClock(clk_period, self.mach_clk_sig)
            self.osch_used = True
        else:
            raise ConstraintError
        return self.mach_clk_sig


class Platform(LatticePlatform):
    default_clk_name = "osch_clk"
    default_clk_period = 1000.0/15.6

    def __init__(self):
        self.osch_clk_period = 1000.0/15.65
        self.osch_routing = OschRouting()    # Internal oscillator routing.
        # Routed during self.do_finalize().
        LatticePlatform.__init__(self, "LCMXO2-1200HC-4SG32C", [],
                                 _connectors, toolchain="diamond")

    # MachXO2 has an internal clock, which is not exposed as an I/O.
    # To get access to it, we handle a request for the clock specially.
    def request(self, *args, **kwargs):
        try:
            sig = GenericPlatform.request(self, *args, **kwargs)
        except ConstraintError:
            if args[0] == "osch_clk":
                # Do not add to self.constraint_manager.matched because we
                # don't want this signal to become part of the UCF.
                sig = self.osch_routing.mk_clk("osch_clk",
                                               self.osch_clk_period)
        return sig

    def do_finalize(self, f, *args, **kwargs):
        # Still add clock to period constraints even though it is not a pin.
        # The constraint reflects the actual frequency chosen plus 5% as a
        # worst case, per MachXO2 oscillator tolerances (+/- 5%, which is
        # unrelated to the +/- 5% error in the frequency selection algorithm).
        # This approach is specified by Lattice:
        # http://www.latticesemi.com/en/Support/AnswerDatabase/4/0/7/4072
        #
        # If the higher of the two closest frequencies to desired was chosen,
        # the constraint will be up to 10% higher than the desired frequency
        # due to the combination of oscillator tolerance and the frequency
        # selection algorithm. If the lower closest frequency was chosen, the
        # constraint will be up to 5% higher than the desired frequency.
        if self.osch_routing.osch_used and hasattr(self, "osch_clk_period"):
            adjusted_freq = 1.05 * \
                self.osch_routing.mclk.nearest_freq(self.osch_clk_period)
            self.add_internal_clock_constraint(self.osch_routing.mach_clk_sig,
                                               1000.0/adjusted_freq)

        # And lastly, add the oscillator routing so the correct primitive
        # is instantiated.
        f += self.osch_routing.get_fragment()

        # Handle cases where OSCH is either not used or not default.
        if self.default_clk_name != "osch_clk":
            GenericPlatform.do_finalize(self, f, *args, **kwargs)

    def set_osch_period(self, period):
        # osch_clk_period is the variable used to set the OSCH period.
        # If default_clk_name == "osch_clk", default_clk_period should _also_
        # be set to osch_clk_period for consistency with the rest of Migen,
        # which may look for a class variable called default_clk_name.
        self.osch_clk_period = period

        if self.default_clk_name == "osch_clk":
            self.default_clk_period = period

    def add_internal_clock_constraint(self, clk, period):
        # Normally clocks are routed from I/O pins and are thus treated as
        # PORTs. However, for internally generated clocks, they need to be
        # specified as NETs.
        self.add_platform_command("""FREQUENCY NET "{{clk}}" {freq} MHz;""".format(freq=float(1/period)*1000, clk="clk"), clk=clk)



# This file is Copyright (c) 2015 Yann Sionneau <yann@sionneau.net>
# This file is Copyright (c) 2015 Florent Kermarrec <florent@enjoy-digital.fr>
# This file is Copyright (c) 2018 William D. Jones <thor0505@comcast.net>
# This file is Copyright (c) 2018 Caleb Jamison <cbjamo@gmail.com>
# License: BSD

from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform, XC3SProg, VivadoProgrammer

_io = [
    ("user_led", 0, Pins("H5"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("J5"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("T9"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("T10"), IOStandard("LVCMOS33")),

    ("rgb_led", 0,
        Subsignal("r", Pins("G6")),
        Subsignal("g", Pins("F6")),
        Subsignal("b", Pins("E1")),
        IOStandard("LVCMOS33"),
    ),

    ("rgb_led", 1,
        Subsignal("r", Pins("G3")),
        Subsignal("g", Pins("J4")),
        Subsignal("b", Pins("G4")),
        IOStandard("LVCMOS33"),
    ),

    ("rgb_led", 2,
        Subsignal("r", Pins("J3")),
        Subsignal("g", Pins("J2")),
        Subsignal("b", Pins("H4")),
        IOStandard("LVCMOS33"),
    ),

    ("rgb_led", 3,
        Subsignal("r", Pins("K1")),
        Subsignal("g", Pins("H6")),
        Subsignal("b", Pins("K2")),
        IOStandard("LVCMOS33"),
    ),

    ("user_sw", 0, Pins("A8"), IOStandard("LVCMOS33")),
    ("user_sw", 1, Pins("C11"), IOStandard("LVCMOS33")),
    ("user_sw", 2, Pins("C10"), IOStandard("LVCMOS33")),
    ("user_sw", 3, Pins("A10"), IOStandard("LVCMOS33")),

    ("user_btn", 0, Pins("D9"), IOStandard("LVCMOS33")),
    ("user_btn", 1, Pins("C9"), IOStandard("LVCMOS33")),
    ("user_btn", 2, Pins("B9"), IOStandard("LVCMOS33")),
    ("user_btn", 3, Pins("B8"), IOStandard("LVCMOS33")),

    ("clk100", 0, Pins("E3"), IOStandard("LVCMOS33")),

    ("cpu_reset", 0, Pins("C2"), IOStandard("LVCMOS33")),

    ("serial", 0,
        Subsignal("tx", Pins("D10")),
        Subsignal("rx", Pins("A9")),
        IOStandard("LVCMOS33")),

    ("spi", 0,
        Subsignal("clk", Pins("F1")),
        Subsignal("cs_n", Pins("C1")),
        Subsignal("mosi", Pins("H1")),
        Subsignal("miso", Pins("G1")),
        IOStandard("LVCMOS33"),
    ),

    ("i2c", 0,
        Subsignal("scl", Pins("L18")),
        Subsignal("sda", Pins("M18")),
        Subsignal("scl_pup", Pins("A14")),
        Subsignal("sda_pup", Pins("A13")),
        IOStandard("LVCMOS33"),
    ),

    ("spiflash4x", 0,  # clock needs to be accessed through STARTUPE2
        Subsignal("cs_n", Pins("L13")),
        Subsignal("dq", Pins("K17", "K18", "L14", "M14")),
        IOStandard("LVCMOS33")
    ),
    ("spiflash", 0,  # clock needs to be accessed through STARTUPE2
        Subsignal("cs_n", Pins("L13")),
        Subsignal("mosi", Pins("K17")),
        Subsignal("miso", Pins("K18")),
        Subsignal("wp", Pins("L14")),
        Subsignal("hold", Pins("M14")),
        IOStandard("LVCMOS33"),
    ),

    ("ddram", 0,
        Subsignal("a", Pins(
            "R2 M6 N4 T1 N6 R7 V6 U7",
            "R8 V7 R6 U6 T6 T8"),
            IOStandard("SSTL135")),
        Subsignal("ba", Pins("R1 P4 P2"), IOStandard("SSTL135")),
        Subsignal("ras_n", Pins("P3"), IOStandard("SSTL135")),
        Subsignal("cas_n", Pins("M4"), IOStandard("SSTL135")),
        Subsignal("we_n", Pins("P5"), IOStandard("SSTL135")),
        Subsignal("cs_n", Pins("U8"), IOStandard("SSTL135")),
        Subsignal("dm", Pins("L1 U1"), IOStandard("SSTL135")),
        Subsignal("dq", Pins(
            "K5 L3 K3 L6 M3 M1 L4 M2",
            "V4 T5 U4 V5 V1 T3 U3 R3"),
            IOStandard("SSTL135"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_p", Pins("N2 U2"), IOStandard("DIFF_SSTL135")),
        Subsignal("dqs_n", Pins("N1 V2"), IOStandard("DIFF_SSTL135")),
        Subsignal("clk_p", Pins("U9"), IOStandard("DIFF_SSTL135")),
        Subsignal("clk_n", Pins("V9"), IOStandard("DIFF_SSTL135")),
        Subsignal("cke", Pins("N5"), IOStandard("SSTL135")),
        Subsignal("odt", Pins("R5"), IOStandard("SSTL135")),
        Subsignal("reset_n", Pins("K6"), IOStandard("SSTL135")),
        Misc("SLEW=FAST"),
    ),

    ("eth_ref_clk", 0, Pins("G18"), IOStandard("LVCMOS33")),
    ("eth_clocks", 0,
        Subsignal("tx", Pins("H16")),
        Subsignal("rx", Pins("F15")),
        IOStandard("LVCMOS33"),
    ),
    ("eth", 0,
        Subsignal("rst_n", Pins("C16")),
        Subsignal("mdio", Pins("K13")),
        Subsignal("mdc", Pins("F16")),
        Subsignal("rx_dv", Pins("G16")),
        Subsignal("rx_er", Pins("C17")),
        Subsignal("rx_data", Pins("D18 E17 E18 G17")),
        Subsignal("tx_en", Pins("H15")),
        Subsignal("tx_data", Pins("H14 J14 J13 H17")),
        Subsignal("col", Pins("D17")),
        Subsignal("crs", Pins("G14")),
        IOStandard("LVCMOS33"),
    ),
]

_connectors = [
    ("pmoda", "G13 B11 A11 D12 D13 B18 A18 K16"),
    ("pmodb", "E15 E16 D15 C15 J17 J18 K15 J15"),
    ("pmodc", "U12 V12 V10 V11 U14 V14 T13 U13"),
    ("pmodd", "D4 D3 F4 F3 E2 D2 H2 G2"),
    ("ck_io", {
        # Outer Digital Header
        "ck_io0" : "V15",
        "ck_io1" : "U16",
        "ck_io2" : "P14",
        "ck_io3" : "T11",
        "ck_io4" : "R12",
        "ck_io5" : "T14",
        "ck_io6" : "T15",
        "ck_io7" : "T16",
        "ck_io8" : "N15",
        "ck_io9" : "M16",
        "ck_io10" : "V17",
        "ck_io11" : "U18",
        "ck_io12" : "R17",
        "ck_io13" : "P17",

        # Inner Digital Header
        "ck_io26" : "U11",
        "ck_io27" : "V16",
        "ck_io28" : "M13",
        "ck_io29" : "R10",
        "ck_io30" : "R11",
        "ck_io31" : "R13",
        "ck_io32" : "R15",
        "ck_io33" : "P15",
        "ck_io34" : "R16",
        "ck_io35" : "N16",
        "ck_io36" : "N14",
        "ck_io37" : "U17",
        "ck_io38" : "T18",
        "ck_io39" : "R18",
        "ck_io40" : "P18",
        "ck_io41" : "N17",

        # Outer Analog Header as Digital IO
        "ck_a0" : "F5",
        "ck_a1" : "D8",
        "ck_a2" : "C7",
        "ck_a3" : "E7",
        "ck_a4" : "D7",
        "ck_a5" : "D5",

        # Inner Analog Header as Digital IO
        "ck_io20" : "B7",
        "ck_io21" : "B6",
        "ck_io22" : "E6",
        "ck_io23" : "E5",
        "ck_io24" : "A4",
        "ck_io25" : "A3",
        } ),
    ("XADC", {
        # Outer Analog Header
        "vaux4_n" : "C5",
        "vaux4_p" : "C6",
        "vaux5_n" : "A5",
        "vaux5_p" : "A6",
        "vaux6_n" : "B4",
        "vaux6_p" : "C4",
        "vaux7_n" : "A1",
        "vaux7_p" : "B1",
        "vaux15_n" : "B2",
        "vaux15_p" : "B3",
        "vaux0_n" : "C14",
        "vaux0_p" : "D14",

        # Inner Analog Header
        "vaux12_n" : "B7",
        "vaux12_p" : "B6",
        "vaux13_n" : "E6",
        "vaux13_p" : "E5",
        "vaux14_n" : "A4",
        "vaux14_p" : "A3",

        # Power Measurements
        "vsnsuv_n" : "B17",
        "vsnsuv_p" : "B16",
        "vsns5v0_n" : "B12",
        "vsns5v0_p" : "C12",
        "isns5v0_n" : "F14",
        "isns5v0_n" : "F13",
        "isns0v95_n" : "A16",
        "isns0v95_n" : "A15",
        } ),
]


class Platform(XilinxPlatform):
    default_clk_name = "clk100"
    default_clk_period = 10.0

    def __init__(self, toolchain="vivado", programmer="vivado"):
        XilinxPlatform.__init__(self, "xc7a35ticsg324-1L", _io, _connectors,
                                toolchain=toolchain)
        self.toolchain.bitstream_commands = \
            ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"]
        self.toolchain.additional_commands = \
            ["write_cfgmem -force -format bin -interface spix4 -size 16 "
             "-loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]
        self.programmer = programmer
        self.add_platform_command("set_property INTERNAL_VREF 0.675 [get_iobanks 34]")

    def create_programmer(self):
        if self.programmer == "xc3sprog":
            return XC3SProg("nexys4")
        elif self.programmer == "vivado":
            return VivadoProgrammer(flash_part="n25q128-3.3v-spi-x1_x2_x4")
        else:
            raise ValueError("{} programmer is not supported"
                             .format(self.programmer))



# This file is Copyright (c) 2016 Ivan Uvarov <flashcactus@ya.ru>
# License: BSD
#
# Note: This file maps only the buttons, switches, LEDs and GPIO headers;
# the actual devboard has much more than that.

from migen.build.generic_platform import *
from migen.build.altera import AlteraPlatform
from migen.build.altera.programmer import USBBlaster


_io = [
    ("clk50", 0, Pins("M9"), IOStandard("3.3-V LVTTL")),
    ("clk50", 1, Pins("H13"), IOStandard("3.3-V LVTTL")),
    ("clk50", 2, Pins("E10"), IOStandard("3.3-V LVTTL")),
    ("clk50", 3, Pins("V15"), IOStandard("3.3-V LVTTL")),

    ("user_led", 0, Pins("AA2"), IOStandard("3.3-V LVTTL")),
    ("user_led", 1, Pins("AA1"), IOStandard("3.3-V LVTTL")),
    ("user_led", 2, Pins("W2"), IOStandard("3.3-V LVTTL")),
    ("user_led", 3, Pins("Y3"), IOStandard("3.3-V LVTTL")),
    ("user_led", 4, Pins("N2"), IOStandard("3.3-V LVTTL")),
    ("user_led", 5, Pins("N1"), IOStandard("3.3-V LVTTL")),
    ("user_led", 6, Pins("U2"), IOStandard("3.3-V LVTTL")),
    ("user_led", 7, Pins("U1"), IOStandard("3.3-V LVTTL")),
    ("user_led", 8, Pins("L2"), IOStandard("3.3-V LVTTL")),
    ("user_led", 9, Pins("L1"), IOStandard("3.3-V LVTTL")),


    ("seven_seg", 0, Pins("U21 V21 W22 W21 Y22 Y21 AA22"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 1, Pins("AA20 AB20 AA19 AA18 AB18 AA17 U22"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 2, Pins("Y19 AB17 AA10 Y14 V14 AB22 AB21"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 3, Pins("Y16 W16 Y17 V16 U17 V18 V19"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 4, Pins("U20 Y20 V20 U16 U15 Y15 P9"), IOStandard("3.3-V LVTTL")),
    ("seven_seg", 5, Pins("N9 M8 T14 P14 C1 C2 W19"), IOStandard("3.3-V LVTTL")),


    ("key", 0, Pins("U7"), IOStandard("3.3-V LVTTL")),
    ("key", 1, Pins("W9"), IOStandard("3.3-V LVTTL")),
    ("key", 2, Pins("M7"), IOStandard("3.3-V LVTTL")),
    ("key", 3, Pins("M6"), IOStandard("3.3-V LVTTL")),

    ("sw", 0, Pins("U13"), IOStandard("3.3-V LVTTL")),
    ("sw", 1, Pins("V13"), IOStandard("3.3-V LVTTL")),
    ("sw", 2, Pins("T13"), IOStandard("3.3-V LVTTL")),
    ("sw", 3, Pins("T12"), IOStandard("3.3-V LVTTL")),
    ("sw", 4, Pins("AA15"), IOStandard("3.3-V LVTTL")),
    ("sw", 5, Pins("AB15"), IOStandard("3.3-V LVTTL")),
    ("sw", 6, Pins("AA14"), IOStandard("3.3-V LVTTL")),
    ("sw", 7, Pins("AA13"), IOStandard("3.3-V LVTTL")),
    ("sw", 8, Pins("AB13"), IOStandard("3.3-V LVTTL")),
    ("sw", 9, Pins("AB12"), IOStandard("3.3-V LVTTL")),


    ("gpio_0", 0,
        Pins("N16 B16 M16 C16 D17 K20 K21 K22 M20 M21 N21 R22 R21 T22 N20 N19 M22 P19 L22 P17 P16 M18 L18 L17 L19 K17 K19 P18 R15 R17 R16 T20 T19 T18 T17 T15"),
        IOStandard("3.3-V LVTTL")
    ),
    ("gpio_1", 0,
        Pins("H16 A12 H15 B12 A13 B13 C13 D13 G18 G17 H18 J18 J19 G11 H10 J11 H14 A15 J13 L8 A14 B15 C15 E14 E15 E16 F14 F15 F13 F12 G16 G15 G13 G12 J17 K16"),
        IOStandard("3.3-V LVTTL")
    ),
]


class Platform(AlteraPlatform):
    default_clk_name = "clk50"
    default_clk_period = 20

    def __init__(self):
        AlteraPlatform.__init__(self, "5CEBA4F23C7", _io)

    def create_programmer(self):
        return USBBlaster()



from migen.fhdl.module import Module
from migen.fhdl.specials import Instance
from migen.genlib.io import DifferentialInput, DifferentialOutput


class AlteraDifferentialInputImpl(Module):
    def __init__(self, i_p, i_n, o):
        self.specials += Instance("ALT_INBUF_DIFF",
                                  name="ibuf_diff",
                                  i_i=i_p,
                                  i_ibar=i_n,
                                  o_o=o)


class AlteraDifferentialInput:
    @staticmethod
    def lower(dr):
        return AlteraDifferentialInputImpl(dr.i_p, dr.i_n, dr.o)


class AlteraDifferentialOutputImpl(Module):
    def __init__(self, i, o_p, o_n):
        self.specials += Instance("ALT_OUTBUF_DIFF",
                                  name="obuf_diff",
                                  i_i=i,
                                  o_o=o_p,
                                  o_obar=o_n)


class AlteraDifferentialOutput:
    @staticmethod
    def lower(dr):
        return AlteraDifferentialOutputImpl(dr.i, dr.o_p, dr.o_n)


altera_special_overrides = {
    DifferentialInput: AlteraDifferentialInput,
    DifferentialOutput: AlteraDifferentialOutput
}



import os
import subprocess
import sys

from migen.fhdl.structure import _Fragment
from migen.build.generic_platform import *
from migen.build import tools
from migen.build.xilinx import common


def _format_constraint(c):
    if isinstance(c, Pins):
        return "LOC=" + c.identifiers[0]
    elif isinstance(c, IOStandard):
        return "IOSTANDARD=" + c.name
    elif isinstance(c, Drive):
        return "DRIVE=" + str(c.strength)
    elif isinstance(c, Misc):
        return c.misc


def _format_ucf(signame, pin, others, resname):
    fmt_c = []
    for c in [Pins(pin)] + others:
        fc = _format_constraint(c)
        if fc is not None:
            fmt_c.append(fc)
    fmt_r = resname[0] + ":" + str(resname[1])
    if resname[2] is not None:
        fmt_r += "." + resname[2]
    return "NET \"" + signame + "\" " + " | ".join(fmt_c) + "; # " + fmt_r + "\n"


def _build_ucf(named_sc, named_pc):
    r = ""
    for sig, pins, others, resname in named_sc:
        if len(pins) > 1:
            for i, p in enumerate(pins):
                r += _format_ucf(sig + "(" + str(i) + ")", p, others, resname)
        else:
            r += _format_ucf(sig, pins[0], others, resname)
    if named_pc:
        r += "\n" + "\n\n".join(named_pc)
    return r


def _build_xst_files(device, sources, vincpaths, build_name, xst_opt):
    prj_contents = ""
    for filename, language, library in sources:
        prj_contents += language + " " + library + " " + tools.cygpath(filename) + "\n"
    tools.write_to_file(build_name + ".prj", prj_contents)

    xst_contents = """run
-ifn {build_name}.prj
-top {build_name}
{xst_opt}
-ofn {build_name}.ngc
-p {device}
""".format(build_name=build_name, xst_opt=xst_opt, device=device)
    if vincpaths:
        xst_contents += "-vlgincdir {"
        for path in vincpaths:
            xst_contents += tools.cygpath(path) + " "
        xst_contents += "}"
    tools.write_to_file(build_name + ".xst", xst_contents)


def _run_yosys(device, sources, vincpaths, build_name):
    ys_contents = ""
    incflags = ""
    for path in vincpaths:
        incflags += " -I" + path
    for filename, language, library in sources:
        ys_contents += "read_{}{} {}\n".format(language, incflags, filename)

    ys_contents += """hierarchy -check -top top
proc; memory; opt; fsm; opt
synth_xilinx -top top -edif {build_name}.edif""".format(build_name=build_name)

    ys_name = build_name + ".ys"
    tools.write_to_file(ys_name, ys_contents)
    r = subprocess.call(["yosys", ys_name])
    if r != 0:
        raise OSError("Subprocess failed")


def _run_ise(build_name, mode, ngdbuild_opt, toolchain, platform):
    if sys.platform == "win32" or sys.platform == "cygwin":
        script_ext = ".bat"
        shell = ["cmd", "/c"]
        build_script_contents = "@echo off\nrem Autogenerated by Migen\n"
        fail_stmt = " || exit /b"
    else:
        script_ext = ".sh"
        shell = ["bash"]
        build_script_contents = "# Autogenerated by Migen\nset -e\n"
        fail_stmt = ""
    if mode == "edif":
        ext = "edif"
    else:
        ext = "ngc"
        build_script_contents += """
xst -ifn {build_name}.xst{fail_stmt}
"""

    build_script_contents += """
ngdbuild {ngdbuild_opt} -uc {build_name}.ucf {build_name}.{ext} {build_name}.ngd{fail_stmt}
"""
    if mode == "cpld":
        build_script_contents += """
cpldfit -ofmt verilog {par_opt} -p {device} {build_name}.ngd{fail_stmt}
taengine -f {build_name}.vm6 -detail -iopath -l {build_name}.tim{fail_stmt}
hprep6 -s IEEE1532 -i {build_name}.vm6{fail_stmt}
"""
    else:
        build_script_contents += """
map {map_opt} -o {build_name}_map.ncd {build_name}.ngd {build_name}.pcf{fail_stmt}
par {par_opt} {build_name}_map.ncd {build_name}.ncd {build_name}.pcf{fail_stmt}
bitgen {bitgen_opt} {build_name}.ncd {build_name}.bit{fail_stmt}
"""
    build_script_contents = build_script_contents.format(build_name=build_name,
            ngdbuild_opt=ngdbuild_opt, bitgen_opt=toolchain.bitgen_opt, ext=ext,
            par_opt=toolchain.par_opt, map_opt=toolchain.map_opt,
            device=platform.device, fail_stmt=fail_stmt)
    build_script_contents += toolchain.ise_commands.format(build_name=build_name)
    build_script_file = "build_" + build_name + script_ext
    tools.write_to_file(build_script_file, build_script_contents, force_unix=False)
    command = shell + [build_script_file]
    r = tools.subprocess_call_filtered(command, common.colors)
    if r != 0:
        raise OSError("Subprocess failed")


class XilinxISEToolchain:
    attr_translate = {
        "keep": ("keep", "true"),
        "no_retiming": ("register_balancing", "no"),
        "async_reg": None,
        "mr_ff": None,
        "ars_ff1": None,
        "ars_ff2": None,
        "no_shreg_extract": ("shreg_extract", "no")
    }

    def __init__(self):
        self.xst_opt = """-ifmt MIXED
-use_new_parser yes
-opt_mode SPEED
-register_balancing yes"""
        self.map_opt = "-ol high -w"
        self.par_opt = "-ol high -w"
        self.ngdbuild_opt = ""
        self.bitgen_opt = "-g Binary:Yes -w"
        self.ise_commands = ""

    def build(self, platform, fragment, build_dir="build", build_name="top",
            run=True, mode="xst", **kwargs):
        if not isinstance(fragment, _Fragment):
            fragment = fragment.get_fragment()

        platform.finalize(fragment)
        ngdbuild_opt = self.ngdbuild_opt
        vns = None

        os.makedirs(build_dir, exist_ok=True)
        cwd = os.getcwd()
        os.chdir(build_dir)
        try:
            if mode in ("xst", "yosys", "cpld"):
                v_output = platform.get_verilog(fragment, name=build_name, **kwargs)
                vns = v_output.ns
                named_sc, named_pc = platform.resolve_signals(vns)
                v_file = build_name + ".v"
                v_output.write(v_file)
                sources = platform.sources | {(v_file, "verilog", "work")}
                if mode in ("xst", "cpld"):
                    _build_xst_files(platform.device, sources, platform.verilog_include_paths, build_name, self.xst_opt)
                    isemode = mode
                else:
                    _run_yosys(platform.device, sources, platform.verilog_include_paths, build_name)
                    isemode = "edif"
                    ngdbuild_opt += "-p " + platform.device

            if mode == "mist":
                from mist import synthesize
                synthesize(fragment, platform.constraint_manager.get_io_signals())

            if mode == "edif" or mode == "mist":
                e_output = platform.get_edif(fragment)
                vns = e_output.ns
                named_sc, named_pc = platform.resolve_signals(vns)
                e_file = build_name + ".edif"
                e_output.write(e_file)
                isemode = "edif"

            tools.write_to_file(build_name + ".ucf", _build_ucf(named_sc, named_pc))
            if run:
                _run_ise(build_name, isemode, ngdbuild_opt, self, platform)
        finally:
            os.chdir(cwd)

        return vns

    # ISE is broken and you must use *separate* TNM_NET objects for period
    # constraints and other constraints otherwise it will be unable to trace
    # them through clock objects like DCM and PLL objects.

    def add_period_constraint(self, platform, clk, period):
        platform.add_platform_command(
            """
NET "{clk}" TNM_NET = "PRD{clk}";
TIMESPEC "TS{clk}" = PERIOD "PRD{clk}" """ + str(period) + """ ns HIGH 50%;
""",
            clk=clk,
            )

    def add_false_path_constraint(self, platform, from_, to):
        platform.add_platform_command(
            """
NET "{from_}" TNM_NET = "TIG{from_}";
NET "{to}" TNM_NET = "TIG{to}";
TIMESPEC "TS{from_}TO{to}" = FROM "TIG{from_}" TO "TIG{to}" TIG;
""",
            from_=from_,
            to=to,
            )



"""
Encoders and decoders between binary and one-hot representation
"""

from migen.fhdl.structure import *
from migen.fhdl.module import Module


class Encoder(Module):
    """Encode one-hot to binary

    If `n` is low, the `o` th bit in `i` is asserted, else none or
    multiple bits are asserted.

    Parameters
    ----------
    width : int
        Bit width of the input

    Attributes
    ----------
    i : Signal(width), in
        One-hot input
    o : Signal(max=width), out
        Encoded binary
    n : Signal(1), out
        Invalid, either none or multiple input bits are asserted
    """
    def __init__(self, width):
        self.i = Signal(width)  # one-hot
        self.o = Signal(max=max(2, width))  # binary
        self.n = Signal()  # invalid: none or multiple
        act = dict((1<<j, self.o.eq(j)) for j in range(width))
        act["default"] = self.n.eq(1)
        self.comb += Case(self.i, act)


class PriorityEncoder(Module):
    """Priority encode requests to binary

    If `n` is low, the `o` th bit in `i` is asserted and the bits below
    `o` are unasserted, else `o == 0`. The LSB has priority.

    Parameters
    ----------
    width : int
        Bit width of the input

    Attributes
    ----------
    i : Signal(width), in
        Input requests
    o : Signal(max=width), out
        Encoded binary
    n : Signal(1), out
        Invalid, no input bits are asserted
    """
    def __init__(self, width):
        self.i = Signal(width)  # one-hot, lsb has priority
        self.o = Signal(max=max(2, width))  # binary
        self.n = Signal()  # none
        for j in range(width)[::-1]:  # last has priority
            self.comb += If(self.i[j], self.o.eq(j))
        self.comb += self.n.eq(self.i == 0)


class Decoder(Module):
    """Decode binary to one-hot

    If `n` is low, the `i` th bit in `o` is asserted, the others are
    not, else `o == 0`.

    Parameters
    ----------
    width : int
        Bit width of the output

    Attributes
    ----------
    i : Signal(max=width), in
        Input binary
    o : Signal(width), out
        Decoded one-hot
    n : Signal(1), in
        Invalid, no output bits are to be asserted
    """

    def __init__(self, width):
        self.i = Signal(max=max(2, width))  # binary
        self.n = Signal()  # none/invalid
        self.o = Signal(width)  # one-hot
        act = dict((j, self.o.eq(1<<j)) for j in range(width))
        self.comb += Case(self.i, act)
        self.comb += If(self.n, self.o.eq(0))


class PriorityDecoder(Decoder):
    pass  # same



import subprocess

from migen.build.generic_programmer import GenericProgrammer


class USBBlaster(GenericProgrammer):
    needs_bitreverse = False

    def __init__(self, cable_name="USB-Blaster", device_id=1):
        self.cable_name = cable_name
        self.device_id = device_id

    def load_bitstream(self, bitstream_file, cable_suffix=""):
        subprocess.call(["quartus_pgm", "-m", "jtag", "-c",
                         "{}{}".format(self.cable_name, cable_suffix), "-o",
                         "p;{}@{}".format(bitstream_file, self.device_id)])



# This file is Copyright (c) 2015 Florent Kermarrec <florent@enjoy-digital.fr>
# License: BSD

import os
import sys
import subprocess
import shutil

from migen.fhdl.structure import _Fragment
from migen.fhdl.verilog import DummyAttrTranslate

from migen.build.generic_platform import *
from migen.build import tools
from migen.build.lattice import common


def _produces_jedec(device):
    return device.startswith("LCMX")


def _format_constraint(c):
    if isinstance(c, Pins):
        return ("LOCATE COMP ", " SITE " + "\"" + c.identifiers[0] + "\"")
    elif isinstance(c, IOStandard):
        return ("IOBUF PORT ", " IO_TYPE=" + c.name)
    elif isinstance(c, Misc):
        return ("IOBUF PORT ", " " + c.misc)


def _format_lpf(signame, pin, others, resname):
    fmt_c = [_format_constraint(c) for c in ([Pins(pin)] + others)]
    r = ""
    for pre, suf in fmt_c:
        r += pre + "\"" + signame + "\"" + suf + ";\n"
    return r


def _build_lpf(named_sc, named_pc):
    r = "BLOCK RESETPATHS;\n"
    r += "BLOCK ASYNCPATHS;\n"
    for sig, pins, others, resname in named_sc:
        if len(pins) > 1:
            for i, p in enumerate(pins):
                r += _format_lpf(sig + "[" + str(i) + "]", p, others, resname)
        else:
            r += _format_lpf(sig, pins[0], others, resname)
    if named_pc:
        r += "\n" + "\n\n".join(named_pc)
    return r


def _build_files(device, sources, vincpaths, build_name):
    tcl = []
    tcl.append("prj_project new -name \"{}\" -impl \"impl\" -dev {} -synthesis \"synplify\"".format(build_name, device))
    for path in vincpaths:
        tcl.append("prj_impl option {include path} {\"" + path + "\"}")
    for filename, language, library in sources:
        tcl.append("prj_src add \"" + filename + "\" -work " + library)
    tcl.append("prj_impl option top \"{}\"".format(build_name))
    tcl.append("prj_project save")
    tcl.append("prj_run Synthesis -impl impl -forceOne")
    tcl.append("prj_run Translate -impl impl")
    tcl.append("prj_run Map -impl impl")
    tcl.append("prj_run PAR -impl impl")
    tcl.append("prj_run Export -impl impl -task Bitgen")
    if _produces_jedec(device):
        tcl.append("prj_run Export -impl impl -task Jedecgen")
    tools.write_to_file(build_name + ".tcl", "\n".join(tcl))


def _build_script(build_name, device):
    if sys.platform in ("win32", "cygwin"):
        script_ext = ".bat"
        build_script_contents = "@echo off\nrem Autogenerated by Migen\n\n"
        copy_stmt = "copy"
        fail_stmt = " || exit /b"
    else:
        script_ext = ".sh"
        build_script_contents = "# Autogenerated by Migen\nset -e\n\n"
        copy_stmt = "cp"
        fail_stmt = ""

    build_script_contents += "pnmainc {tcl_script}{fail_stmt}\n".format(
        tcl_script=build_name + ".tcl",
        fail_stmt=fail_stmt)
    for ext in (".bit", ".jed"):
        if ext == ".jed" and not _produces_jedec(device):
            continue

        diamond_product = os.path.join("impl", build_name + "_impl" + ext)
        if sys.platform in ("win32", "cygwin"):
            # While Windows itself has no problems with forward slashes
            # in paths, the COPY command on Windows uses forward slash for
            # arguments. MinGW Python 3 may (it is not consistent) create
            # paths with forward slashes on Windows, so remove them just in
            # case.
            diamond_product = diamond_product.replace("/", "\\")

        build_script_contents += "{copy_stmt} {diamond_product} {migen_product}" \
                                 "{fail_stmt}\n".format(
            copy_stmt=copy_stmt,
            fail_stmt=fail_stmt,
            diamond_product=diamond_product,
            migen_product=build_name + ext)

    build_script_file = "build_" + build_name + script_ext
    tools.write_to_file(build_script_file, build_script_contents,
                        force_unix=False)
    return build_script_file


def _run_script(script):
    if sys.platform in ("win32", "cygwin"):
        shell = ["cmd", "/c"]
    else:
        shell = ["bash"]

    if subprocess.call(shell + [script]) != 0:
        raise OSError("Subprocess failed")


class LatticeDiamondToolchain:
    attr_translate = {
        # FIXME: document
        "keep": ("syn_keep", "true"),
        "no_retiming": ("syn_no_retiming", "true"),
        "async_reg": None,
        "mr_ff": None,
        "mr_false_path": None,
        "ars_ff1": None,
        "ars_ff2": None,
        "ars_false_path": None,
        "no_shreg_extract": None
    }

    special_overrides = common.lattice_ecpx_special_overrides

    def build(self, platform, fragment, build_dir="build", build_name="top",
              run=True, **kwargs):
        os.makedirs(build_dir, exist_ok=True)
        cwd = os.getcwd()
        os.chdir(build_dir)

        if not isinstance(fragment, _Fragment):
            fragment = fragment.get_fragment()
        platform.finalize(fragment)

        v_output = platform.get_verilog(fragment, name=build_name, **kwargs)
        named_sc, named_pc = platform.resolve_signals(v_output.ns)
        v_file = build_name + ".v"
        v_output.write(v_file)
        sources = platform.sources | {(v_file, "verilog", "work")}
        _build_files(platform.device, sources, platform.verilog_include_paths, build_name)

        tools.write_to_file(build_name + ".lpf", _build_lpf(named_sc, named_pc))

        script = _build_script(build_name, platform.device)
        if run:
            _run_script(script)

        os.chdir(cwd)

        return v_output.ns

    def add_period_constraint(self, platform, clk, period):
        # TODO: handle differential clk
        platform.add_platform_command("""FREQUENCY PORT "{clk}" {freq} MHz;""".format(freq=str(float(1/period)*1000), clk="{clk}"), clk=clk)



# This file is Copyright (c) 2014 Florent Kermarrec <florent@enjoy-digital.fr>
# License: BSD

import os
import subprocess
import sys

from migen.fhdl.structure import _Fragment
from migen.build.generic_platform import *
from migen.build import tools
from migen.build.xilinx import common


def _format_constraint(c):
    if isinstance(c, Pins):
        return "set_property LOC " + c.identifiers[0]
    elif isinstance(c, IOStandard):
        return "set_property IOSTANDARD " + c.name
    elif isinstance(c, Drive):
        return "set_property DRIVE " + str(c.strength)
    elif isinstance(c, Misc):
        return "set_property " + c.misc.replace("=", " ")
    else:
        raise ValueError("unknown constraint {}".format(c))


def _format_xdc(signame, resname, *constraints):
    fmt_c = [_format_constraint(c) for c in constraints]
    fmt_r = resname[0] + ":" + str(resname[1])
    if resname[2] is not None:
        fmt_r += "." + resname[2]
    r = " ## {}\n".format(fmt_r)
    for c in fmt_c:
        r += c + " [get_ports " + signame + "]\n"
    return r


def _build_xdc(named_sc, named_pc):
    r = ""
    for sig, pins, others, resname in named_sc:
        if len(pins) > 1:
            for i, p in enumerate(pins):
                r += _format_xdc(sig + "[" + str(i) + "]", resname, Pins(p), *others)
        elif pins:
            r += _format_xdc(sig, resname, Pins(pins[0]), *others)
        else:
            r += _format_xdc(sig, resname, *others)
    if named_pc:
        r += "\n" + "\n\n".join(named_pc)
    return r


def _run_vivado(build_name):
    if sys.platform == "win32" or sys.platform == "cygwin":
        build_script_contents = "REM Autogenerated by Migen\n"
        build_script_contents += "vivado -mode batch -source " + build_name + ".tcl\n"
        build_script_file = "build_" + build_name + ".bat"
        tools.write_to_file(build_script_file, build_script_contents)
        command = build_script_file
    else:
        build_script_contents = "# Autogenerated by Migen\nset -e\n"
        build_script_contents += "export LC_ALL=C\n"  # work around "locale::facet::_S_create_c_locale name not valid"
        build_script_contents += "vivado -mode batch -source " + build_name + ".tcl\n"
        build_script_file = "build_" + build_name + ".sh"
        tools.write_to_file(build_script_file, build_script_contents)
        command = ["bash", build_script_file]
    r = tools.subprocess_call_filtered(command, common.colors)
    if r != 0:
        raise OSError("Subprocess failed")


class XilinxVivadoToolchain:
    attr_translate = {
        "keep": ("dont_touch", "true"),
        "no_retiming": ("dont_touch", "true"),
        "async_reg": ("async_reg", "true"),
        "mr_ff": ("mr_ff", "true"),  # user-defined attribute
        "ars_ff1": ("ars_ff1", "true"),  # user-defined attribute
        "ars_ff2": ("ars_ff2", "true"),  # user-defined attribute
        "no_shreg_extract": None
    }

    def __init__(self):
        self.bitstream_commands = []
        self.additional_commands = []
        self.pre_synthesis_commands = []
        self.with_phys_opt = False
        self.clocks = dict()
        self.false_paths = set()

    def _build_batch(self, platform, sources, edifs, ips, build_name):
        tcl = []
        tcl.append("create_project -force -name {} -part {}".format(build_name, platform.device))
        tcl.append("set_property XPM_LIBRARIES {XPM_CDC XPM_MEMORY} [current_project]")
        for filename, language, library in sources:
            filename_tcl = "{" + filename + "}"
            tcl.append("add_files " + filename_tcl)
            tcl.append("set_property library {} [get_files {}]"
                       .format(library, filename_tcl))

        for filename in edifs:
            filename_tcl = "{" + filename + "}"
            tcl.append("read_edif " + filename_tcl)

        for filename in ips:
            filename_tcl = "{" + filename + "}"
            ip = os.path.splitext(os.path.basename(filename))[0]
            tcl.append("read_ip " + filename_tcl)
            tcl.append("upgrade_ip [get_ips {}]".format(ip))
            tcl.append("generate_target all [get_ips {}]".format(ip))
            tcl.append("synth_ip [get_ips {}] -force".format(ip))
            tcl.append("get_files -all -of_objects [get_files {}]".format(filename_tcl))

        tcl.append("read_xdc {}.xdc".format(build_name))
        tcl.extend(c.format(build_name=build_name) for c in self.pre_synthesis_commands)
        # "-include_dirs {}" crashes Vivado 2016.4
        if platform.verilog_include_paths:
            tcl.append("synth_design -top {} -part {} -include_dirs {{{}}}".format(build_name, platform.device, " ".join(platform.verilog_include_paths)))
        else:
            tcl.append("synth_design -top {} -part {}".format(build_name, platform.device))
        tcl.append("report_timing_summary -file {}_timing_synth.rpt".format(build_name))
        tcl.append("report_utilization -hierarchical -file {}_utilization_hierarchical_synth.rpt".format(build_name))
        tcl.append("report_utilization -file {}_utilization_synth.rpt".format(build_name))
        tcl.append("opt_design")
        tcl.append("place_design")
        if self.with_phys_opt:
            tcl.append("phys_opt_design -directive AddRetime")
        tcl.append("report_utilization -hierarchical -file {}_utilization_hierarchical_place.rpt".format(build_name))
        tcl.append("report_utilization -file {}_utilization_place.rpt".format(build_name))
        tcl.append("report_io -file {}_io.rpt".format(build_name))
        tcl.append("report_control_sets -verbose -file {}_control_sets.rpt".format(build_name))
        tcl.append("report_clock_utilization -file {}_clock_utilization.rpt".format(build_name))
        tcl.append("route_design")
        tcl.append("phys_opt_design")
        tcl.append("report_timing_summary -no_header -no_detailed_paths")
        tcl.append("write_checkpoint -force {}_route.dcp".format(build_name))
        tcl.append("report_route_status -file {}_route_status.rpt".format(build_name))
        tcl.append("report_drc -file {}_drc.rpt".format(build_name))
        tcl.append("report_timing_summary -datasheet -max_paths 10 -file {}_timing.rpt".format(build_name))
        tcl.append("report_power -file {}_power.rpt".format(build_name))
        for bitstream_command in self.bitstream_commands:
            tcl.append(bitstream_command.format(build_name=build_name))
        tcl.append("write_bitstream -force {}.bit ".format(build_name))
        for additional_command in self.additional_commands:
            tcl.append(additional_command.format(build_name=build_name))
        tcl.append("quit")
        tools.write_to_file(build_name + ".tcl", "\n".join(tcl))

    def _convert_clocks(self, platform):
        for clk, period in sorted(self.clocks.items(), key=lambda x: x[0].duid):
            platform.add_platform_command(
                "create_clock -name {clk} -period " + str(period) +
                " [get_nets {clk}]", clk=clk)
        for from_, to in sorted(self.false_paths,
                                key=lambda x: (x[0].duid, x[1].duid)):
            platform.add_platform_command(
                "set_clock_groups "
                "-group [get_clocks -include_generated_clocks -of [get_nets {from_}]] "
                "-group [get_clocks -include_generated_clocks -of [get_nets {to}]] "
                "-asynchronous",
                from_=from_, to=to)

        # make sure add_*_constraint cannot be used again
        del self.clocks
        del self.false_paths

    def _constrain(self, platform):
        # The asynchronous input to a MultiReg is a false path
        platform.add_platform_command(
            "set_false_path -quiet "
            "-to [get_nets -filter {{mr_ff == TRUE}}]"
        )
        # The asychronous reset input to the AsyncResetSynchronizer is a false
        # path
        platform.add_platform_command(
            "set_false_path -quiet "
            "-to [get_pins -filter {{REF_PIN_NAME == PRE}} "
                "-of [get_cells -filter {{ars_ff1 == TRUE || ars_ff2 == TRUE}}]]"
        )
        # clock_period-2ns to resolve metastability on the wire between the
        # AsyncResetSynchronizer FFs
        platform.add_platform_command(
            "set_max_delay 2 -quiet "
            "-from [get_pins -filter {{REF_PIN_NAME == Q}} "
                "-of [get_cells -filter {{ars_ff1 == TRUE}}]] "
            "-to [get_pins -filter {{REF_PIN_NAME == D}} "
                "-of [get_cells -filter {{ars_ff2 == TRUE}}]]"
        )

    def build(self, platform, fragment, build_dir="build", build_name="top",
            run=True, **kwargs):
        os.makedirs(build_dir, exist_ok=True)
        cwd = os.getcwd()
        os.chdir(build_dir)

        if not isinstance(fragment, _Fragment):
            fragment = fragment.get_fragment()
        platform.finalize(fragment)
        self._convert_clocks(platform)
        self._constrain(platform)
        v_output = platform.get_verilog(fragment, name=build_name, **kwargs)
        named_sc, named_pc = platform.resolve_signals(v_output.ns)
        v_file = build_name + ".v"
        v_output.write(v_file)
        sources = platform.sources | {(v_file, "verilog", "work")}
        edifs = platform.edifs
        ips = platform.ips
        self._build_batch(platform, sources, edifs, ips, build_name)
        tools.write_to_file(build_name + ".xdc", _build_xdc(named_sc, named_pc))
        if run:
            _run_vivado(build_name)

        os.chdir(cwd)

        return v_output.ns

    def add_period_constraint(self, platform, clk, period):
        if clk in self.clocks:
            raise ValueError("A period constraint already exists")
        self.clocks[clk] = period

    def add_false_path_constraint(self, platform, from_, to):
        if (to, from_) not in self.false_paths:
            self.false_paths.add((from_, to))



import inspect
from sys import version_info
from opcode import opname
from collections import defaultdict

# All opcodes are 2 bytes in length in Python 3.6
def _bytecode_length_version_guard(old_len):
    return old_len if version_info[1] < 6 else 2

_call_opcodes = {
    "CALL_FUNCTION" : _bytecode_length_version_guard(3),
    "CALL_FUNCTION_KW" : _bytecode_length_version_guard(3),
}

if version_info[1] < 6:
    _call_opcodes["CALL_FUNCTION_VAR"] = 3
    _call_opcodes["CALL_FUNCTION_VAR_KW"] = 3
elif version_info[1] < 7:
    _call_opcodes["CALL_FUNCTION_EX"] = 2
else:
    _call_opcodes["CALL_FUNCTION_EX"] = 2
    _call_opcodes["CALL_METHOD"] = 2
    _call_opcodes["CALL"] = 2

_load_build_opcodes = {
    "LOAD_GLOBAL" : _bytecode_length_version_guard(3),
    "LOAD_NAME" : _bytecode_length_version_guard(3),
    "LOAD_ATTR" : _bytecode_length_version_guard(3),
    "LOAD_FAST" : _bytecode_length_version_guard(3),
    "LOAD_DEREF" : _bytecode_length_version_guard(3),
    "DUP_TOP" : _bytecode_length_version_guard(1),
    "BUILD_LIST" : _bytecode_length_version_guard(3),
    "CACHE" : _bytecode_length_version_guard(3),
    "COPY" : _bytecode_length_version_guard(3),
}


def get_var_name(frame):
    code = frame.f_code
    call_index = frame.f_lasti
    while call_index > 0 and opname[code.co_code[call_index]] == "CACHE":
        call_index -= 2
    while True:
        call_opc = opname[code.co_code[call_index]]
        if call_opc in("EXTENDED_ARG",):
            call_index += 2
        else:
            break
    if call_opc not in _call_opcodes:
        return None
    index = call_index+_call_opcodes[call_opc]
    while True:
        opc = opname[code.co_code[index]]
        if opc == "STORE_NAME" or opc == "STORE_ATTR":
            name_index = int(code.co_code[index+1])
            return code.co_names[name_index]
        elif opc == "STORE_FAST":
            name_index = int(code.co_code[index+1])
            return code.co_varnames[name_index]
        elif opc == "STORE_DEREF":
            name_index = int(code.co_code[index+1])
            if version_info >= (3, 11):
                name_index -= code.co_nlocals
            return code.co_cellvars[name_index]
        elif opc in _load_build_opcodes:
            index += _load_build_opcodes[opc]
        else:
            return None


def remove_underscore(s):
    if len(s) > 2 and s[0] == "_" and s[1] != "_":
        s = s[1:]
    return s


def get_obj_var_name(override=None, default=None):
    if override:
        return override

    frame = inspect.currentframe().f_back
    # We can be called via derived classes. Go back the stack frames
    # until we reach the first class that does not inherit from us.
    ourclass = frame.f_locals["self"].__class__
    while "self" in frame.f_locals and isinstance(frame.f_locals["self"], ourclass):
        frame = frame.f_back

    vn = get_var_name(frame)
    if vn is None:
        vn = default
    else:
        vn = remove_underscore(vn)
    return vn

name_to_idx = defaultdict(int)
classname_to_objs = dict()


def index_id(l, obj):
    for n, e in enumerate(l):
        if id(e) == id(obj):
            return n
    raise ValueError


def trace_back(varname=None):
    l = []
    frame = inspect.currentframe().f_back.f_back
    while frame is not None:
        if varname is None:
            varname = get_var_name(frame)
        if varname is not None:
            varname = remove_underscore(varname)
            l.insert(0, (varname, name_to_idx[varname]))
            name_to_idx[varname] += 1

        try:
            obj = frame.f_locals["self"]
        except KeyError:
            obj = None
        if hasattr(obj, "__del__"):
            obj = None

        if obj is None:
            if varname is not None:
                coname = frame.f_code.co_name
                if coname == "<module>":
                    modules = frame.f_globals["__name__"]
                    modules = modules.split(".")
                    coname = modules[len(modules)-1]
                coname = remove_underscore(coname)
                l.insert(0, (coname, name_to_idx[coname]))
                name_to_idx[coname] += 1
        else:
            classname = obj.__class__.__name__.lower()
            try:
                objs = classname_to_objs[classname]
            except KeyError:
                classname_to_objs[classname] = [obj]
                idx = 0
            else:
                try:
                    idx = index_id(objs, obj)
                except ValueError:
                    idx = len(objs)
                    objs.append(obj)
            classname = remove_underscore(classname)
            l.insert(0, (classname, idx))

        varname = None
        frame = frame.f_back
    return l



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform
from migen.build.xilinx.programmer import UrJTAG


_io = [
    ("user_led", 0, Pins("V5"), IOStandard("LVCMOS33"), Drive(24), Misc("SLEW=QUIETIO")),

    ("clk50", 0, Pins("AB13"), IOStandard("LVCMOS33")),

    # When executing softcore code in-place from the flash, we want
    # the flash reset to be released before the system reset.
    ("norflash_rst_n", 0, Pins("P22"), IOStandard("LVCMOS33"), Misc("SLEW=FAST"), Drive(8)),
    ("norflash", 0,
        Subsignal("adr", Pins("L22 L20 K22 K21 J19 H20 F22",
            "F21 K17 J17 E22 E20 H18 H19 F20",
            "G19 C22 C20 D22 D21 F19 F18 D20 D19")),
        Subsignal("d", Pins("AA20 U14 U13 AA6 AB6 W4 Y4 Y7",
            "AA2 AB2 V15 AA18 AB18 Y13 AA12 AB12"), Misc("PULLDOWN")),
        Subsignal("oe_n", Pins("M22")),
        Subsignal("we_n", Pins("N20")),
        Subsignal("ce_n", Pins("M21")),
        IOStandard("LVCMOS33"), Misc("SLEW=FAST"), Drive(8)
    ),

    ("serial", 0,
        Subsignal("tx", Pins("L17"), IOStandard("LVCMOS33"), Misc("SLEW=SLOW")),
        Subsignal("rx", Pins("K18"), IOStandard("LVCMOS33"), Misc("PULLUP"))
    ),

    ("ddram_clock", 0,
        Subsignal("p", Pins("M3")),
        Subsignal("n", Pins("L4")),
        IOStandard("SSTL2_I")
    ),
    ("ddram", 0,
        Subsignal("a", Pins("B1 B2 H8 J7 E4 D5 K7 F5 G6 C1 C3 D1 D2")),
        Subsignal("ba", Pins("A2 E6")),
        Subsignal("cs_n", Pins("F7")),
        Subsignal("cke", Pins("G7")),
        Subsignal("ras_n", Pins("E5")),
        Subsignal("cas_n", Pins("C4")),
        Subsignal("we_n", Pins("D3")),
        Subsignal("dq", Pins("Y2 W3 W1 P8 P7 P6 P5 T4 T3",
            "U4 V3 N6 N7 M7 M8 R4 P4 M6 L6 P3 N4",
            "M5 V2 V1 U3 U1 T2 T1 R3 R1 P2 P1")),
        Subsignal("dm", Pins("E1 E3 F3 G4")),
        Subsignal("dqs", Pins("F1 F2 H5 H6")),
        IOStandard("SSTL2_I")
    ),

    ("eth_clocks", 0,
        Subsignal("phy", Pins("M20")),
        Subsignal("rx", Pins("H22")),
        Subsignal("tx", Pins("H21")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 0,
        Subsignal("rst_n", Pins("R22")),
        Subsignal("rx_dv", Pins("V21")),
        Subsignal("rx_er", Pins("V22")),
        Subsignal("rx_data", Pins("U22 U20 T22 T21")),
        Subsignal("tx_en", Pins("N19")),
        Subsignal("tx_er", Pins("M19")),
        Subsignal("tx_data", Pins("M16 L15 P19 P20")),
        Subsignal("col", Pins("W20")),
        Subsignal("crs", Pins("W22")),
        IOStandard("LVCMOS33")
    ),

    ("vga_out", 0,
        Subsignal("clk", Pins("A10")),
        Subsignal("r", Pins("C6 B6 A6 C7 A7 B8 A8 D9")),
        Subsignal("g", Pins("C8 C9 A9 D7 D8 D10 C10 B10")),
        Subsignal("b", Pins("D11 C12 B12 A12 C13 A13 D14 C14")),
        Subsignal("hsync_n", Pins("A14")),
        Subsignal("vsync_n", Pins("C15")),
        Subsignal("psave_n", Pins("B14")),
        IOStandard("LVCMOS33")
    ),
    ("dvi_out", 0,
        Subsignal("clk_p", Pins("W12"), IOStandard("TMDS_33")),
        Subsignal("clk_n", Pins("Y12"), IOStandard("TMDS_33")),
        Subsignal("data0_p", Pins("Y16"), IOStandard("TMDS_33")),
        Subsignal("data0_n", Pins("W15"), IOStandard("TMDS_33")),
        Subsignal("data1_p", Pins("AA16"), IOStandard("TMDS_33")),
        Subsignal("data1_n", Pins("AB16"), IOStandard("TMDS_33")),
        Subsignal("data2_p", Pins("Y15"), IOStandard("TMDS_33")),
        Subsignal("data2_n", Pins("AB15"), IOStandard("TMDS_33")),
    ),

    ("mmc", 0,
        Subsignal("clk", Pins("J3")),
        Subsignal("cmd", Pins("K1")),
        Subsignal("dat", Pins("J6 K6 N1 K5")),
        IOStandard("LVCMOS33")
    ),

    ("dvi_in", 0,
        Subsignal("clk_p", Pins("K20"), IOStandard("TMDS_33")),
        Subsignal("clk_n", Pins("K19"), IOStandard("TMDS_33")),
        Subsignal("data0_p", Pins("B21"), IOStandard("TMDS_33")),
        Subsignal("data0_n", Pins("B22"), IOStandard("TMDS_33")),
        Subsignal("data1_p", Pins("A20"), IOStandard("TMDS_33")),
        Subsignal("data1_n", Pins("A21"), IOStandard("TMDS_33")),
        Subsignal("data2_p", Pins("K16"), IOStandard("TMDS_33")),
        Subsignal("data2_n", Pins("J16"), IOStandard("TMDS_33")),
        Subsignal("scl", Pins("G20"), IOStandard("LVCMOS33")),
        Subsignal("sda", Pins("H16"), IOStandard("LVCMOS33")),
        Subsignal("hpd_notif", Pins("G22"), IOStandard("LVCMOS33")),
        Subsignal("hpd_en", Pins("G17"), IOStandard("LVCMOS33"))
    ),
    ("dvi_in", 1,
        Subsignal("clk_p", Pins("C11"), IOStandard("TMDS_33")),
        Subsignal("clk_n", Pins("A11"), IOStandard("TMDS_33")),
        Subsignal("data0_p", Pins("B18"), IOStandard("TMDS_33")),
        Subsignal("data0_n", Pins("A18"), IOStandard("TMDS_33")),
        Subsignal("data1_p", Pins("C17"), IOStandard("TMDS_33")),
        Subsignal("data1_n", Pins("A17"), IOStandard("TMDS_33")),
        Subsignal("data2_p", Pins("E16"), IOStandard("TMDS_33")),
        Subsignal("data2_n", Pins("D17"), IOStandard("TMDS_33")),
        Subsignal("scl", Pins("F17"), IOStandard("LVCMOS33")),
        Subsignal("sda", Pins("F16"), IOStandard("LVCMOS33")),
        Subsignal("hpd_notif", Pins("G16"), IOStandard("LVCMOS33")),
        Subsignal("hpd_en", Pins("B20"), IOStandard("LVCMOS33"))
    ),
    ("dvi_in", 2,
        Subsignal("clk_p", Pins("Y11"), IOStandard("TMDS_33")),
        Subsignal("clk_n", Pins("AB11"), IOStandard("TMDS_33")),
        Subsignal("data0_p", Pins("V11"), IOStandard("TMDS_33")),
        Subsignal("data0_n", Pins("W11"), IOStandard("TMDS_33")),
        Subsignal("data1_p", Pins("AA10"), IOStandard("TMDS_33")),
        Subsignal("data1_n", Pins("AB10"), IOStandard("TMDS_33")),
        Subsignal("data2_p", Pins("R11"), IOStandard("TMDS_33")),
        Subsignal("data2_n", Pins("T11"), IOStandard("TMDS_33")),
        Subsignal("scl", Pins("C16"), IOStandard("LVCMOS33")),
        Subsignal("sda", Pins("B16"), IOStandard("LVCMOS33")),
        Subsignal("hpd_notif", Pins("D6"), IOStandard("LVCMOS33")),
        Subsignal("hpd_en", Pins("A4"), IOStandard("LVCMOS33"))
    ),
    ("dvi_in", 3,
        Subsignal("clk_p", Pins("J20"), IOStandard("TMDS_33")),
        Subsignal("clk_n", Pins("J22"), IOStandard("TMDS_33")),
        Subsignal("data0_p", Pins("P18"), IOStandard("TMDS_33")),
        Subsignal("data0_n", Pins("R19"), IOStandard("TMDS_33")),
        Subsignal("data1_p", Pins("P17"), IOStandard("TMDS_33")),
        Subsignal("data1_n", Pins("N16"), IOStandard("TMDS_33")),
        Subsignal("data2_p", Pins("M17"), IOStandard("TMDS_33")),
        Subsignal("data2_n", Pins("M18"), IOStandard("TMDS_33")),
        Subsignal("scl", Pins("P21"), IOStandard("LVCMOS33")),
        Subsignal("sda", Pins("N22"), IOStandard("LVCMOS33")),
        Subsignal("hpd_notif", Pins("H17"), IOStandard("LVCMOS33")),
        Subsignal("hpd_en", Pins("C19"), IOStandard("LVCMOS33"))
    ),
]


class Platform(XilinxPlatform):
    default_clk_name = "clk50"
    default_clk_period = 20

    def __init__(self):
        XilinxPlatform.__init__(self, "xc6slx45-fgg484-2", _io)
        self.add_platform_command("CONFIG VCCAUX=\"3.3\";\n")

    def create_programmer(self):
        return UrJTAG("fjmem-mixxeo.bit")

    def do_finalize(self, fragment):
        XilinxPlatform.do_finalize(self, fragment)

        try:
            eth_clocks = self.lookup_request("eth_clocks")
            self.add_period_constraint(eth_clocks.rx, 40)
            self.add_period_constraint(eth_clocks.tx, 40)
            self.add_platform_command("""
TIMESPEC "TS{phy_tx_clk}_io" = FROM "GRP{phy_tx_clk}" TO "PADS" 10 ns;
TIMESPEC "TS{phy_rx_clk}_io" = FROM "PADS" TO "GRP{phy_rx_clk}" 10 ns;
""", phy_rx_clk=eth_clocks.rx, phy_tx_clk=eth_clocks.tx)
        except ConstraintError:
            pass

        for i in range(4):
            try:
                self.add_period_constraint(self.lookup_request("dvi_in", i).clk_p, 12)
            except ConstraintError:
                pass



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


# Bank 34 and 35 voltage depend on J18 jumper setting
_io = [
        ("clk100", 0, Pins("Y9"), IOStandard("LVCMOS33")),

        ("user_btn", 0, Pins("P16"), IOStandard("LVCMOS18")),  # center
        ("user_btn", 1, Pins("R16"), IOStandard("LVCMOS18")),  # down
        ("user_btn", 2, Pins("N15"), IOStandard("LVCMOS18")),  # left
        ("user_btn", 3, Pins("R18"), IOStandard("LVCMOS18")),  # right
        ("user_btn", 4, Pins("T18"), IOStandard("LVCMOS18")),  # up

        ("user_sw", 0, Pins("F22"), IOStandard("LVCMOS18")),
        ("user_sw", 1, Pins("G22"), IOStandard("LVCMOS18")),
        ("user_sw", 2, Pins("H22"), IOStandard("LVCMOS18")),
        ("user_sw", 3, Pins("F21"), IOStandard("LVCMOS18")),
        ("user_sw", 4, Pins("H19"), IOStandard("LVCMOS18")),
        ("user_sw", 5, Pins("H18"), IOStandard("LVCMOS18")),
        ("user_sw", 6, Pins("H17"), IOStandard("LVCMOS18")),
        ("user_sw", 7, Pins("M15"), IOStandard("LVCMOS18")),

        ("user_led", 0, Pins("T22"), IOStandard("LVCMOS33")),
        ("user_led", 1, Pins("T21"), IOStandard("LVCMOS33")),
        ("user_led", 2, Pins("U22"), IOStandard("LVCMOS33")),
        ("user_led", 3, Pins("U21"), IOStandard("LVCMOS33")),
        ("user_led", 4, Pins("V22"), IOStandard("LVCMOS33")),
        ("user_led", 5, Pins("W22"), IOStandard("LVCMOS33")),
        ("user_led", 6, Pins("U19"), IOStandard("LVCMOS33")),
        ("user_led", 7, Pins("U14"), IOStandard("LVCMOS33")),

        # A
        ("pmod", 0, Pins("Y11 AA11 Y10 AA9 AB11 AB10 AB9 AA8"),
            IOStandard("LVCMOS33")),
        # B
        ("pmod", 1, Pins("W12 W11 V10 W8 V12 W10 V9 V8"),
            IOStandard("LVCMOS33")),
        # C
        ("pmod", 2,
            Subsignal("n", Pins("AB6 AA4 T6 U4")),
            Subsignal("p", Pins("AB7 Y4 R6 T4")),
            IOStandard("LVCMOS33")),
        # D
        ("pmod", 3,
            Subsignal("n", Pins("W7 V4 W5 U5")),
            Subsignal("p", Pins("V7 V5 W6 U6")),
            IOStandard("LVCMOS33")),

        ("audio", 0,
            Subsignal("adr", Pins("AB1 Y5")),
            Subsignal("gpio", Pins("Y8 AA7 AA6 Y6")),
            Subsignal("mclk", Pins("AB2")),
            Subsignal("sck", Pins("AB4")),
            Subsignal("sda", Pins("AB5")),
            IOStandard("LVCMOS33")),

        ("oled", 0,
            Subsignal("dc", Pins("U10")),
            Subsignal("res", Pins("U9")),
            Subsignal("sclk", Pins("AB12")),
            Subsignal("sdin", Pins("AA12")),
            Subsignal("vbat", Pins("U11")),
            Subsignal("vdd", Pins("U12")),
            IOStandard("LVCMOS33")),

        ("hdmi", 0,
            Subsignal("clk", Pins("W18")),
            Subsignal("d", Pins(
                "Y13 AA13 AA14 Y14 AB15 AB16 AA16 AB17 "
                "AA17 Y15 W13 W15 V15 U17 V14 V13")),
            Subsignal("de", Pins("U16")),
            Subsignal("hsync", Pins("V17")),
            Subsignal("vsync", Pins("W17")),
            Subsignal("int", Pins("W16")),
            Subsignal("scl", Pins("AA18")),
            Subsignal("sda", Pins("Y16")),
            Subsignal("spdif", Pins("U15")),
            Subsignal("spdifo", Pins("Y18")),
            IOStandard("LVCMOS33")),

        ("netic16", 0,
            Subsignal("w20", Pins("W20")),
            Subsignal("w21", Pins("W21")),
            IOStandard("LVCMOS33")),

        ("vga", 0,
            Subsignal("r", Pins("V20 U20 V19 V18")),
            Subsignal("g", Pins("AB22 AA22 AB21 AA21")),
            Subsignal("b", Pins("Y21 Y20 AB20 AB19")),
            Subsignal("hsync_n", Pins("AA19")),
            Subsignal("vsync_n", Pins("Y19")),
            IOStandard("LVCMOS33")),

        ("usb_otg", 0,
            Subsignal("vbusoc", Pins("L16")),
            Subsignal("reset_n", Pins("G17")),
            IOStandard("LVCMOS18")),

        ("pudc_b", 0, Pins("K16"), IOStandard("LVCMOS18")),

        ("xadc", 0,
            Subsignal("gio", Pins("H15 R15 K15 J15")),
            Subsignal("ad0_n", Pins("E16")),
            Subsignal("ad0_p", Pins("F16")),
            Subsignal("ad8_n", Pins("D17")),
            Subsignal("ad8_p", Pins("D16")),
            IOStandard("LVCMOS18")),

        ("fmc_clocks", 0,
            Subsignal("clk0_n", Pins("L19")),
            Subsignal("clk0_p", Pins("L18")),
            Subsignal("clk1_n", Pins("C19")),
            Subsignal("clk1_p", Pins("D18")),
            IOStandard("LVCMOS18")),

        ("fmc", 0,
            Subsignal("scl", Pins("R7")),
            Subsignal("sda", Pins("U7")),

            Subsignal("prsnt", Pins("AB14")),

            # 0, 1, 17, 18 can be clock signals
            Subsignal("la_n", Pins(
                "M20 N20 P18 P22 M22 K18 L22 T17 "
                "J22 R21 T19 N18 P21 M17 K20 J17 "
                "K21 B20 C20 G16 G21 E20 F19 D15 "
                "A19 C22 E18 D21 A17 C18 B15 B17 "
                "A22 B22")),
            Subsignal("la_p", Pins(
                "M19 N19 P17 N22 M21 J18 L21 T16 "
                "J21 R20 R19 N17 P20 L17 K19 J16 "
                "J20 B19 D20 G15 G20 E19 G19 E15 "
                "A18 D22 F18 E21 A16 C17 C15 B16 "
                "A21 B21")),
            IOStandard("LVCMOS18")),
]


class Platform(XilinxPlatform):
    default_clk_name = "clk100"
    default_clk_period = 10

    def __init__(self):
        XilinxPlatform.__init__(self, "xc7z020-clg484-1", _io, toolchain="vivado")



import subprocess

from migen.build.generic_programmer import GenericProgrammer


class OpenOCD(GenericProgrammer):
    needs_bitreverse = False

    def __init__(self, config, flash_proxy_basename=None):
        GenericProgrammer.__init__(self, flash_proxy_basename)
        self.config = config

    def load_bitstream(self, bitstream):
        script = "; ".join([
            "init",
            "pld load 0 {{{}}}".format(bitstream),
            "exit",
        ])
        subprocess.call(["openocd", "-f", self.config, "-c", script])

    def flash(self, address, data):
        flash_proxy = self.find_flash_proxy()
        script = "; ".join([
            "init",
            "jtagspi_init 0 {{{}}}".format(flash_proxy),
            "jtagspi_program {{{}}} 0x{:x}".format(data, address),
            "fpga_program",
            "exit"
        ])
        subprocess.call(["openocd", "-f", self.config, "-c", script])



from migen.fhdl.structure import *
from migen.fhdl.module import Module
from migen.fhdl.tools import insert_reset, rename_clock_domain


__all__ = ["CEInserter", "ResetInserter", "ClockDomainsRenamer",
           "ModuleTransformer"]


class ModuleTransformer:
    # overload this in derived classes
    def transform_instance(self, i):
        pass

    # overload this in derived classes
    def transform_fragment(self, i, f):
        pass

    def wrap_class(self, victim):
        class Wrapped(victim):
            def __init__(i, *args, **kwargs):
                victim.__init__(i, *args, **kwargs)
                self.transform_instance(i)

            def get_fragment(i):
                f = victim.get_fragment(i)
                self.transform_fragment(i, f)
                return f

        Wrapped.__name__ = victim.__name__
        Wrapped.__doc__ = victim.__doc__
        Wrapped.__module__ = victim.__module__
        return Wrapped

    def wrap_instance(self, victim):
        self.transform_instance(victim)
        orig_get_fragment = victim.get_fragment

        def get_fragment():
            f = orig_get_fragment()
            self.transform_fragment(victim, f)
            return f

        victim.get_fragment = get_fragment
        return victim

    def __call__(self, victim):
        if isinstance(victim, Module):
            return self.wrap_instance(victim)
        else:
            return self.wrap_class(victim)


class ControlInserter(ModuleTransformer):
    control_name = None  # override this

    def __init__(self, clock_domains=None):
        self.clock_domains = clock_domains

    def transform_instance(self, i):
        if self.clock_domains is None:
            ctl = Signal(name=self.control_name)
            assert not hasattr(i, self.control_name)
            setattr(i, self.control_name, ctl)
        else:
            for cd in self.clock_domains:
                name = self.control_name + "_" + cd
                ctl = Signal(name=name)
                assert not hasattr(i, name)
                setattr(i, name, ctl)

    def transform_fragment(self, i, f):
        if self.clock_domains is None:
            if not f.sync:
                return
            if len(f.sync) > 1:
                raise ValueError("Control signal clock domains must be specified when module has more than one domain")
            cdn = list(f.sync.keys())[0]
            to_insert = [(getattr(i, self.control_name), cdn)]
        else:
            to_insert = [(getattr(i, self.control_name + "_" + cdn), cdn)
                for cdn in self.clock_domains]
        self.transform_fragment_insert(i, f, to_insert)


class CEInserter(ControlInserter):
    control_name = "ce"

    def transform_fragment_insert(self, i, f, to_insert):
        for ce, cdn in to_insert:
            f.sync[cdn] = [If(ce, *f.sync[cdn])]


class ResetInserter(ControlInserter):
    control_name = "reset"

    def transform_fragment_insert(self, i, f, to_insert):
        for reset, cdn in to_insert:
            f.sync[cdn] = insert_reset(reset, f.sync[cdn])


class ClockDomainsRenamer(ModuleTransformer):
    def __init__(self, cd_remapping):
        if isinstance(cd_remapping, str):
            cd_remapping = {"sys": cd_remapping}
        self.cd_remapping = cd_remapping

    def transform_fragment(self, i, f):
        for old, new in self.cd_remapping.items():
            rename_clock_domain(f, old, new)



from migen.fhdl.structure import *
from migen.fhdl.module import Module


class Divider(Module):
    def __init__(self, w):
        self.start_i = Signal()
        self.dividend_i = Signal(w)
        self.divisor_i = Signal(w)
        self.ready_o = Signal()
        self.quotient_o = Signal(w)
        self.remainder_o = Signal(w)

        ###

        qr = Signal(2*w)
        counter = Signal(max=w+1)
        divisor_r = Signal(w)
        diff = Signal(w+1)

        self.comb += [
            self.quotient_o.eq(qr[:w]),
            self.remainder_o.eq(qr[w:]),
            self.ready_o.eq(counter == 0),
            diff.eq(qr[w-1:] - divisor_r)
        ]
        self.sync += [
            If(self.start_i,
                counter.eq(w),
                qr.eq(self.dividend_i),
                divisor_r.eq(self.divisor_i)
            ).Elif(~self.ready_o,
                    If(diff[w],
                        qr.eq(Cat(0, qr[:2*w-1]))
                    ).Else(
                        qr.eq(Cat(1, qr[:w-1], diff[:w]))
                    ),
                    counter.eq(counter - 1)
            )
        ]



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform, VivadoProgrammer, iMPACT
from migen.build.xilinx.ise import XilinxISEToolchain

_io = [
    ("user_led", 0, Pins("M26"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("T24"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("T25"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("R26"), IOStandard("LVCMOS33")),

    ("cpu_reset", 0, Pins("U4"), IOStandard("LVCMOS15")),

    ("user_btn_c", 0, Pins("U6"), IOStandard("SSTL15")),
    ("user_btn_n", 0, Pins("P6"), IOStandard("LVCMOS15")),
    ("user_btn_s", 0, Pins("T5"), IOStandard("SSTL15")),
    ("user_btn_w", 0, Pins("R5"), IOStandard("SSTL15")),
    ("user_btn_e", 0, Pins("U5"), IOStandard("SSTL15")),

    ("user_dip_btn", 0, Pins("R8"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 1, Pins("P8"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 2, Pins("R7"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 3, Pins("R6"), IOStandard("LVCMOS25")),

    ("user_sma_clock", 0,
        Subsignal("p", Pins("J23"), IOStandard("LVDS_25"),
            Misc("DIFF_TERM=TRUE")),
        Subsignal("n", Pins("H23"), IOStandard("LVDS_25"),
            Misc("DIFF_TERM=TRUE"))
    ),
    ("user_sma_clock_p", 0, Pins("J23"), IOStandard("LVCMOS25")),
    ("user_sma_clock_n", 0, Pins("H23"), IOStandard("LVCMOS25")),

    ("user_sma_gpio_p", 0, Pins("T8"), IOStandard("LVCMOS15")),
    ("user_sma_gpio_n", 0, Pins("T7"), IOStandard("LVCMOS15")),

    ("clk200", 0,
        Subsignal("p", Pins("R3"), IOStandard("LVDS_25")),
        Subsignal("n", Pins("P3"), IOStandard("LVDS_25"))
    ),

    ("clk156", 0,
        Subsignal("p", Pins("M21"), IOStandard("LVDS_25")),
        Subsignal("n", Pins("M22"), IOStandard("LVDS_25"))
    ),

    ("i2c", 0,
        Subsignal("scl", Pins("N18")),
        Subsignal("sda", Pins("K25")),
        IOStandard("LVCMOS33")),

    ("serial", 0,
        Subsignal("cts", Pins("W19")),
        Subsignal("rts", Pins("V19")),
        Subsignal("tx", Pins("U19")),
        Subsignal("rx", Pins("T19")),
        IOStandard("LVCMOS18")),

    ("spiflash", 0,  # clock needs to be accessed through STARTUPE2
        Subsignal("cs_n", Pins("P18")),
        Subsignal("dq", Pins("R14", "R15", "P14", "N14")),
        IOStandard("LVCMOS33")
    ),

    ("mmc", 0,
        Subsignal("wp", Pins("R20")),
        Subsignal("det", Pins("P24")),
        Subsignal("cmd", Pins("N23")),
        Subsignal("clk", Pins("N24")),
        Subsignal("dat", Pins("P19 N19 P23 P21")),
        IOStandard("LVCMOS33")),

    ("mmc_spi", 0,
        Subsignal("miso", Pins("P19"), Misc("PULLUP")),
        Subsignal("clk", Pins("N24")),
        Subsignal("mosi", Pins("N23")),
        Subsignal("cs_n", Pins("P21")),
        IOStandard("LVCMOS33")),

    ("lcd", 0,
        Subsignal("db", Pins("L25 M24 M25 L22")),
        Subsignal("e", Pins("L20")),
        Subsignal("rs", Pins("L23")),
        Subsignal("rw", Pins("L24")),
        IOStandard("LVCMOS15")),

    ("rotary", 0,
        Subsignal("a", Pins("N22")),
        Subsignal("b", Pins("P20")),
        Subsignal("push", Pins("N21")),
        IOStandard("LVCMOS33")),

    ("hdmi", 0,
        Subsignal("d", Pins(
            "AA24 Y25 Y26 V26 W26 W25 W24 U26",
            "U25 V24 U20 W23 W20 U24 Y20 V23",
            "AA23 AA25 AB25 AC24 AB24 Y22 Y23 V22")),
        Subsignal("de", Pins("AB26")),
        Subsignal("clk", Pins("V21")),
        Subsignal("vsync", Pins("AC26")),
        Subsignal("hsync", Pins("AA22")),
        Subsignal("int", Pins("W21")),
        Subsignal("spdif", Pins("Y21")),
        Subsignal("spdif_out", Pins("T20")),
        IOStandard("LVCMOS18")),

    ("ddram", 0,
        Subsignal("a", Pins(
            "M4 J3 J1 L4 K5 M7 K1 M6",
            "H1 K3 N7 L5 L7 N6 L3 K2"),
            IOStandard("SSTL15")),
        Subsignal("ba", Pins("N1 M1 H2"), IOStandard("SSTL15")),
        Subsignal("ras_n", Pins("P1"), IOStandard("SSTL15")),
        Subsignal("cas_n", Pins("T4"), IOStandard("SSTL15")),
        Subsignal("we_n", Pins("R1"), IOStandard("SSTL15")),
        Subsignal("cs_n", Pins("T3"), IOStandard("SSTL15")),
        Subsignal("dm", Pins("AC6 AC4 AA3 U7 G1 F3 G5 H9"),
            IOStandard("SSTL15")),
        Subsignal("dq", Pins(
            "AB6 AA8 Y8 AB5 AA5 Y5 Y6 Y7",
            "AF4 AF5 AF3 AE3 AD3 AC3 AB4 AA4",
            "AC2 AB2 AF2 AE2 Y1 Y2 AC1 AB1",
            "Y3 W3 W6 V6 W4 W5 W1 V1",
            "G2 D1 E1 E2 F2 A2 A3 C2",
            "C3 D3 A4 B4 C4 D4 D5 E5",
            "F4 G4 K6 K7 K8 L8 J5 J6",
            "G6 H6 F7 F8 G8 H8 D6 E6"),
            IOStandard("SSTL15")),
        Subsignal("dqs_p", Pins("V8 AD5 AD1 V3 C1 B5 J4 H7"),
            IOStandard("DIFF_SSTL15")),
        Subsignal("dqs_n", Pins("W8 AE5 AE1 V2 B1 A5 H4 G7"),
            IOStandard("DIFF_SSTL15")),
        Subsignal("clk_p", Pins("M2"), IOStandard("DIFF_SSTL15")),
        Subsignal("clk_n", Pins("L2"), IOStandard("DIFF_SSTL15")),
        Subsignal("cke", Pins("P4"), IOStandard("SSTL15")),
        Subsignal("odt", Pins("R2"), IOStandard("SSTL15")),
        Subsignal("reset_n", Pins("N8"), IOStandard("LVCMOS15")),
        Misc("SLEW=FAST"),
    ),

    ("eth_clocks", 0,
        Subsignal("tx", Pins("U22")),
        Subsignal("rx", Pins("U21")),
        IOStandard("LVCMOS18")
    ),
    ("eth", 0,
        Subsignal("rst_n", Pins("V18"), IOStandard("LVCMOS18")),
        Subsignal("mdio", Pins("T14"), IOStandard("LVCMOS18")),
        Subsignal("mdc", Pins("W18"), IOStandard("LVCMOS18")),
        Subsignal("rx_data", Pins("U17 V17 V16 V14"), IOStandard("HSTL_I_18")),
        Subsignal("rx_ctrl", Pins("U14"), IOStandard("HSTL_I_18")),
        Subsignal("tx_data", Pins("U16 U15 T18 T17"), IOStandard("HSTL_I_18")),
        Subsignal("tx_ctrl", Pins("T15"), IOStandard("HSTL_I_18"))
    ),

    ("pcie_x1", 0,
        Subsignal("rst_n", Pins("M20"), IOStandard("LVCMOS33")),
        Subsignal("clk_p", Pins("F11")),
        Subsignal("clk_n", Pins("E11")),
        Subsignal("rx_p", Pins("D12")),
        Subsignal("rx_n", Pins("C12")),
        Subsignal("tx_p", Pins("D10")),
        Subsignal("tx_n", Pins("C10"))
    ),
    ("pcie_x2", 0,
        Subsignal("rst_n", Pins("M20"), IOStandard("LVCMOS33")),
        Subsignal("clk_p", Pins("F11")),
        Subsignal("clk_n", Pins("E11")),
        Subsignal("rx_p", Pins("D12 B13")),
        Subsignal("rx_n", Pins("C12 A13")),
        Subsignal("tx_p", Pins("D10 B9")),
        Subsignal("tx_n", Pins("C10 A9"))
    ),
    ("pcie_x4", 0,
        Subsignal("rst_n", Pins("M20"), IOStandard("LVCMOS33")),
        Subsignal("clk_p", Pins("F11")),
        Subsignal("clk_n", Pins("E11")),
        Subsignal("rx_p", Pins("D12 B13 D14 B11")),
        Subsignal("rx_n", Pins("C12 A13 C14 A11")),
        Subsignal("tx_p", Pins("D10 B9 D8 B7")),
        Subsignal("tx_n", Pins("C10 A9 C8 A7"))
    ),

    ("vadj_on_b", 0, Pins("R16"), IOStandard("LVCMOS33")),

    ("sfp_mgt_clk0", 0,
        Subsignal("p", Pins("AA13")),
        Subsignal("n", Pins("AB13"))
    ),
    ("sfp_mgt_clk1", 0,
        Subsignal("p", Pins("AA11")),
        Subsignal("n", Pins("AB11"))
    ),
    ("user_sma_mgt_tx", 0,
        Subsignal("p", Pins("AE7")),
        Subsignal("n", Pins("AF7"))
    ),
    ("user_sma_mgt_rx", 0,
        Subsignal("p", Pins("AE11")),
        Subsignal("n", Pins("AF11"))
    ),
    ("sfp_tx", 0,  # inverted prior to HW rev 1.1
        Subsignal("p", Pins("AC10")),
        Subsignal("n", Pins("AD10"))
    ),
    ("sfp_rx", 0,  # inverted prior to HW rev 1.1
        Subsignal("p", Pins("AC12")),
        Subsignal("n", Pins("AD12"))
    ),
    ("sfp_tx_disable_n", 0, Pins("R18"), IOStandard("LVCMOS33")),
    ("sfp_rx_los", 0, Pins("R23"), IOStandard("LVCMOS33")),

    ("si5324", 0,
        Subsignal("rst_n", Pins("AB24"), IOStandard("LVCMOS25")),
        Subsignal("int", Pins("M19"), IOStandard("LVCMOS33"))
    ),
    ("si5324_clkin", 0,
        Subsignal("p", Pins("D23"), IOStandard("LVDS_25")),
        Subsignal("n", Pins("D24"), IOStandard("LVDS_25"))
    ),
    # Not included pin definitions:
    # - PMBUS
    # - Fan PWM and Tach
    # - XADC Mux address signals
    # - CLK multiplexer selection address signals
]


_connectors = [
    ("HPC", {
        # SerDes with up to 6.6Gbd, no defined IOStandard
        "DP0_C2M_P": "AE9",
        "DP0_C2M_N": "AF9",
        "DP0_M2C_P": "AE13",
        "DP0_M2C_N": "AF13",
        "DP1_C2M_P": "AC8",
        "DP1_C2M_N": "AD8",
        "DP1_M2C_P": "AC14",
        "DP1_M2C_N": "AD14",
        # These CLK signals are not directly connected to the FPGA.
        # One can use them by configuring the clock multiplexers
        # and take the CLK signal at sfp_mgt_clk0 and/or sfp_mgt_clk1
        #"GBTCLK1_M2C_P": "",
        #"GBTCLK1_M2C_N": "",
        #"GBTCLK0_M2C_P": "",
        #"GBTCLK0_M2C_N": "",

        # All these pins are located on banks with a single adjustable voltage.
        # By default this voltage is 2.5V.
        # The IOStandard has to be configured as needed
        # with the used supply voltage in mind.
        "LA06_P": "G19",
        "LA06_N": "F20",
        "LA10_P": "A17",
        "LA10_N": "A18",
        "LA14_P": "C21",
        "LA14_N": "B21",
        "LA18_CC_P": "G20",
        "LA18_CC_N": "G21",
        "LA27_P": "F23",
        "LA27_N": "E23",
        "HA01_CC_P": "AB21",
        "HA01_CC_N": "AC21",
        "HA05_P": "AD25",
        "HA05_N": "AD26",
        "HA09_P": "AF19",
        "HA09_N": "AF20",
        "HA13_P": "AC18",
        "HA13_N": "AD18",
        "HA16_P": "AE17",
        "HA16_N": "AF17",
        "HA20_P": "Y16",
        "HA20_N": "Y17",
        "CLK1_M2C_P": "H21",
        "CLK1_M2C_N": "H22",
        "LA00_CC_P": "D18",
        "LA00_CC_N": "C18",
        "LA03_P": "G17",
        "LA03_N": "F17",
        "LA08_P": "C17",
        "LA08_N": "B17",
        "LA12_P": "E20",
        "LA12_N": "D20",
        "LA16_P": "E21",
        "LA16_N": "D21",
        "LA20_P": "M16",
        "LA20_N": "M17",
        "LA22_P": "L17",
        "LA22_N": "L18",
        "LA25_P": "G22",
        "LA25_N": "F22",
        "LA29_P": "G24",
        "LA29_N": "F24",
        "LA31_P": "E26",
        "LA31_N": "D26",
        "LA33_P": "G25",
        "LA33_N": "F25",
        "HA03_P": "AC22",
        "HA03_N": "AC23",
        "HA07_P": "AD23",
        "HA07_N": "AD24",
        "HA11_P": "AD20",
        "HA11_N": "AE20",
        "HA14_P": "AE18",
        "HA14_N": "AF18",
        "HA18_P": "AA17",
        "HA18_N": "AB17",
        "HA22_P": "Y15",
        "HA22_N": "AA15",
        "LA01_CC_P": "E17",
        "LA01_CC_N": "E18",
        "LA05_P": "G15",
        "LA05_N": "F15",
        "LA09_P": "E16",
        "LA09_N": "D16",
        "LA13_P": "B20",
        "LA13_N": "A20",
        "LA17_CC_P": "K21",
        "LA17_CC_N": "J21",
        "LA23_P": "K20",
        "LA23_N": "J20",
        "LA26_P": "J24",
        "LA26_N": "H24",
        "PG_M2C": "N17",
        "HA00_CC_P": "AA19",
        "HA00_CC_N": "AB19",
        "HA04_P": "AF24",
        "HA04_N": "AF25",
        "HA08_P": "AD21",
        "HA08_N": "AE21",
        "HA12_P": "AC19",
        "HA12_N": "AD19",
        "HA15_P": "Y18",
        "HA15_N": "AA18",
        "HA19_P": "AC17",
        "HA19_N": "AD17",
        "PRSNT_M2C_B": "N16",
        "CLK0_M2C_P": "D19",
        "CLK0_M2C_N": "C19",
        "LA02_P": "H14",
        "LA02_N": "H15",
        "LA04_P": "F18",
        "LA04_N": "F19",
        "LA07_P": "H16",
        "LA07_N": "G16",
        "LA11_P": "B19",
        "LA11_N": "A19",
        "LA15_P": "B22",
        "LA15_N": "A22",
        "LA19_P": "M14",
        "LA19_N": "L14",
        "LA21_P": "J19",
        "LA21_N": "H19",
        "LA24_P": "J18",
        "LA24_N": "H18",
        "LA28_P": "K22",
        "LA28_N": "K23",
        "LA30_P": "E25",
        "LA30_N": "D25",
        "LA32_P": "H26",
        "LA32_N": "G26",
        "HA02_P": "AE25",
        "HA02_N": "AE26",
        "HA06_P": "AE23",
        "HA06_N": "AF23",
        "HA10_P": "AE22",
        "HA10_N": "AF22",
        "HA17_CC_P": "AA20",
        "HA17_CC_N": "AB20",
        "HA21_P": "AB16",
        "HA21_N": "AC16",
        "HA23_P": "W14",
        "HA23_N": "W15",
        }
    ),
    ("XADC", {
        # GPIOs are located on banks with a single adjustable voltage.
        # By default this voltage is 2.5V.
        # The IOStandard has to be configured as needed
        # with the used supply voltage in mind.
        "GPIO0": "H17",
        "GPIO1": "E22",
        "GPIO2": "K18",
        "GPIO3": "L19",
        # Voltage restrictions of the XADC do apply.
        "VAUX0_N": "J16",
        "VAUX0_P": "K15",
        "VAUX8_N": "J15",
        "VAUX8_P": "J14",
        }
    ),
    # Not included connector definitions:
    # - PMOD
]


class Platform(XilinxPlatform):
    default_clk_name = "clk156"
    default_clk_period = 6.4

    def __init__(self, toolchain="vivado", programmer="vivado"):
        XilinxPlatform.__init__(self, "xc7a200t-fbg676-2", _io, _connectors,
            toolchain=toolchain)
        if toolchain == "ise":
            self.toolchain.bitgen_opt = "-g LCK_cycle:6 -g Binary:Yes -w -g ConfigRate:12 -g SPI_buswidth:4"
        elif toolchain == "vivado":
            self.toolchain.bitstream_commands = ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"]
            self.toolchain.additional_commands = ["write_cfgmem -force -format bin -interface spix4 -size 16 -loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]
        self.programmer = programmer

    def create_programmer(self):
        if self.programmer == "vivado":
            return VivadoProgrammer()
        elif self.programmer == "impact":
            return iMPACT()
        else:
            raise ValueError("{} programmer is not supported".format(programmer))

    def do_finalize(self, fragment):
        XilinxPlatform.do_finalize(self, fragment)
        try:
            self.add_period_constraint(self.lookup_request("clk200").p, 5.0)
        except ConstraintError:
            pass
        try:
            self.add_period_constraint(self.lookup_request("eth_clocks").rx, 8.0)
        except ConstraintError:
            pass
        try:
            self.add_period_constraint(self.lookup_request("eth_clocks").tx, 8.0)
        except ConstraintError:
            pass



# This file is Copyright (c) 2016-2017 William D. Jones <thor0505@comcast.net>
# License: BSD

import os
import sys
import subprocess

from migen.fhdl.structure import _Fragment

from migen.build.generic_platform import *
from migen.build import tools
from migen.build.lattice import common


def _build_pcf(named_sc, named_pc):
    r = ""
    for sig, pins, others, resname in named_sc:
        if len(pins) > 1:
            for bit, pin in enumerate(pins):
                r += "set_io {}[{}] {}\n".format(sig, bit, pin)
        else:
            r += "set_io {} {}\n".format(sig, pins[0])
    if named_pc:
        r += "\n" + "\n\n".join(named_pc)
    return r


def _build_pre_pack(vns, freq_cstrs):
    r = ""
    for sig in freq_cstrs:
        r += """ctx.addClock("{}", {})\n""".format(vns.get_name(sig), freq_cstrs[sig])
    return r


def _build_script(source, build_template, build_name, **kwargs):
    if sys.platform in ("win32", "cygwin"):
        script_ext = ".bat"
        build_script_contents = "@echo off\nrem Autogenerated by Migen\n\n"
        fail_stmt = " || exit /b"
    else:
        script_ext = ".sh"
        build_script_contents = "# Autogenerated by Migen\nset -e\n\n"
        fail_stmt = ""

    for s in build_template:
        s_fail = s + "{fail_stmt}\n"  # Required so Windows scripts fail early.
        build_script_contents += s_fail.format(build_name=build_name,
                                               fail_stmt=fail_stmt,
                                               **kwargs)

    build_script_file = "build_" + build_name + script_ext
    tools.write_to_file(build_script_file, build_script_contents,
                        force_unix=False)
    return build_script_file


def _run_script(script):
    if sys.platform in ("win32", "cygwin"):
        shell = ["cmd", "/c"]
    else:
        shell = ["bash"]

    if subprocess.call(shell + [script]) != 0:
        raise OSError("Subprocess failed")


class LatticeIceStormToolchain:
    attr_translate = {
        "keep": ("keep", "true"),
        "no_retiming": None,  # yosys can do retiming via the (non-default)
                              # "-retime" option to "synth_ice40", but
                              # yosys does not check for an equivalent
                              # constraint to prevent retiming on signals.
        "async_reg": None,  # yosys has no equivalent, and arachne-pnr
                            # wouldn't take advantage of it anyway.

        # While custom attributes are supported in yosys, neither
        # arachne-pnr nor icetime currently can take advantage of them
        # to add fine-grained timing constraints.
        "mr_ff": None,  # user-defined attribute
        "mr_false_path": None,  # user-defined attribute
        "ars_ff1": None,  # user-defined attribute
        "ars_ff2": None,  # user-defined attribute
        "ars_false_path": None,  # user-defined attribute

        # ice40 does not have a shift register primitive.
        "no_shreg_extract": None
    }

    special_overrides = common.lattice_ice40_special_overrides

    def __init__(self):
        # Variables within replacement fields should be backend-aware and
        # update their syntax accordingly. Currently, only {pnr_pkg_opts}
        # needs this functionality.

        self.yosys_template = [
            "{read_files}",
            "attrmap -tocase keep -imap keep=\"true\" keep=1 -imap keep=\"false\" keep=0 -remove keep=0",
            "synth_ice40 -top {build_name} -blif {build_name}.blif",
        ]

        self.build_template = [
            "yosys -q -l {build_name}.rpt {build_name}.ys",
            "arachne-pnr -q -l {pnr_pkg_opts} -p {build_name}.pcf {build_name}.blif -o {build_name}.txt",
            "icetime {icetime_pkg_opts} -c {freq_constraint} -t -p {build_name}.pcf -r {build_name}.tim {build_name}.txt",
            "icepack {build_name}.txt {build_name}.bin"
        ]

        self.nextpnr_yosys_template = [
            "{read_files}",
            "attrmap -tocase keep -imap keep=\"true\" keep=1 -imap keep=\"false\" keep=0 -remove keep=0",
            "synth_ice40 {synth_opts} -top {build_name} -json {build_name}.json",
        ]

        self.nextpnr_build_template = [
            "yosys -q -l {build_name}.rpt {build_name}.ys",
            "nextpnr-ice40 {pnr_pkg_opts} --pcf {build_name}.pcf --json {build_name}.json --asc {build_name}.txt --pre-pack {build_name}_pre_pack.py",
            "icepack {build_name}.txt {build_name}.bin"
        ]

        self.freq_constraints = dict()

    # platform.device should be of the form "ice40-{lp384, hx1k, etc}-{tq144, etc}"
    def build(self, platform, fragment, build_dir="build", build_name="top",
              use_nextpnr=True, synth_opts="", run=True, **kwargs):
        os.makedirs(build_dir, exist_ok=True)
        cwd = os.getcwd()
        os.chdir(build_dir)

        if not isinstance(fragment, _Fragment):
            fragment = fragment.get_fragment()
        platform.finalize(fragment)

        v_output = platform.get_verilog(fragment, name=build_name, **kwargs)
        named_sc, named_pc = platform.resolve_signals(v_output.ns)
        v_file = build_name + ".v"
        v_output.write(v_file)

        if use_nextpnr:
            chosen_yosys_template = self.nextpnr_yosys_template
        else:
            chosen_yosys_template = self.yosys_template
        ys_contents = "\n".join(_.format(build_name=build_name,
                                         read_files=self.gen_read_files(platform, v_file),
                                         synth_opts=synth_opts)
                                for _ in chosen_yosys_template)

        ys_name = build_name + ".ys"
        tools.write_to_file(ys_name, ys_contents)

        tools.write_to_file(build_name + ".pcf",
                            _build_pcf(named_sc, named_pc))
        (family, series_size, package) = self.parse_device_string(platform.device)
        if use_nextpnr:
            pnr_pkg_opts = "--" + series_size + " --package " + package
        else:
            pnr_pkg_opts = "-d " + self.get_size_string(series_size) + \
                           " -P " + package
        icetime_pkg_opts = "-P " + package + " -d " + series_size

        if use_nextpnr:
            tools.write_to_file(build_name + "_pre_pack.py",
                                _build_pre_pack(v_output.ns, self.freq_constraints))
        # icetime can only handle a single global constraint, so we test against the fastest
        # clock; though imprecise, if the global design satisfies the fastest clock, we can
        # be sure all other constraints are satisfied.
        freq_constraint = str(max(self.freq_constraints.values(),
                                  default=0.0))

        if use_nextpnr:
            chosen_build_template = self.nextpnr_build_template
        else:
            chosen_build_template = self.build_template
        script = _build_script(source=False,
                               build_template=chosen_build_template,
                               build_name=build_name,
                               pnr_pkg_opts=pnr_pkg_opts,
                               icetime_pkg_opts=icetime_pkg_opts,
                               freq_constraint=freq_constraint)

        if run:
            _run_script(script)

        os.chdir(cwd)

        return v_output.ns

    def parse_device_string(self, device_str):
        # Arachne only understands packages based on the device size, but
        # LP for a given size supports packages that HX for the same size
        # doesn't and vice versa; we need to know the device series due to
        # icetime.
        valid_packages = {
            "lp384": ["qn32", "cm36", "cm49"],
            "lp1k": ["swg16tr", "cm36", "cm49", "cm81", "cb81", "qn84",
                     "cm121", "cb121"],
            "hx1k": ["vq100", "cb132", "tq144"],
            "lp8k": ["cm81", "cm81:4k", "cm121", "cm121:4k", "cm225",
                     "cm225:4k"],
            "hx8k": ["bg121", "bg121:4k", "cb132", "cb132:4k", "cm121",
                     "cm121:4k", "cm225", "cm225:4k", "cm81", "cm81:4k",
                     "ct256", "tq144:4k"],
            "up3k": ["sg48", "uwg30"],
            "up5k": ["sg48", "uwg30"],
        }

        (family, series_size, package) = device_str.split("-")
        if family not in ["ice40"]:
            raise ValueError("Unknown device family")
        if series_size not in ["lp384", "lp1k", "hx1k", "lp8k", "hx8k", "up5k"]:
            raise ValueError("Invalid device series/size")
        if package not in valid_packages[series_size]:
            raise ValueError("Invalid device package")
        return (family, series_size, package)

    def get_size_string(self, series_size_str):
        return series_size_str[2:]

    def gen_read_files(self, platform, main):
        sources = platform.sources | {(main, "verilog", "work")}
        incflags = ""
        read_files = list()
        for path in platform.verilog_include_paths:
            incflags += " -I" + path
        for filename, language, library in sources:
            read_files.append("read_{}{} {}".format(language,
                                                    incflags,
                                                    filename))
        return "\n".join(read_files)

    def add_period_constraint(self, platform, clk, period):
        new_freq = 1000.0/period

        if clk not in self.freq_constraints.keys():
            self.freq_constraints[clk] = new_freq
        else:
            raise ConstraintError("Period constraint already added to signal.")



from migen.build.generic_platform import *
from migen.build.lattice import LatticePlatform
from migen.build.lattice.programmer import IceBurnProgrammer


_io = [
    ("user_led", 0, Pins("59"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("56"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("53"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("51"), IOStandard("LVCMOS33")),

    ("user_btn", 0, Pins("60"), IOStandard("LVCMOS33")),
    ("user_btn", 1, Pins("57"), IOStandard("LVCMOS33")),
    ("user_btn", 2, Pins("54"), IOStandard("LVCMOS33")),
    ("user_btn", 3, Pins("52"), IOStandard("LVCMOS33")),

    ("clk3p3", 0, Pins("13"), IOStandard("LVCMOS33"))
]


class Platform(LatticePlatform):
    default_clk_name = "clk3p3"
    default_clk_period = 303.0303

    def __init__(self):
        LatticePlatform.__init__(self, "ice40-hx1k-vq100", _io,
                                 toolchain="icestorm")

    def create_programmer(self, iceburn_path="./iCEburn.py"):
        return IceBurnProgrammer(iceburn_path)



from migen.fhdl.structure import *
from migen.fhdl.structure import _Slice, _Part, _Assign, _Fragment
from migen.fhdl.visit import NodeVisitor, NodeTransformer
from migen.fhdl.bitcontainer import value_bits_sign
from migen.util.misc import flat_iteration


class _SignalLister(NodeVisitor):
    def __init__(self):
        self.output_list = set()

    def visit_Signal(self, node):
        self.output_list.add(node)


class _TargetLister(NodeVisitor):
    def __init__(self):
        self.output_list = set()
        self.target_context = False

    def visit_Signal(self, node):
        if self.target_context:
            self.output_list.add(node)

    def visit_Assign(self, node):
        self.target_context = True
        self.visit(node.l)
        self.target_context = False

    def visit_ArrayProxy(self, node):
        for choice in node.choices:
            self.visit(choice)


class _InputLister(NodeVisitor):
    def __init__(self):
        self.output_list = set()

    def visit_Signal(self, node):
        self.output_list.add(node)

    def visit_Assign(self, node):
        self.visit(node.r)


def list_signals(node):
    lister = _SignalLister()
    lister.visit(node)
    return lister.output_list


def list_targets(node):
    lister = _TargetLister()
    lister.visit(node)
    return lister.output_list


def list_inputs(node):
    lister = _InputLister()
    lister.visit(node)
    return lister.output_list


def _resort_statements(ol):
    return [statement for i, statement in
            sorted(ol, key=lambda x: x[0])]


def group_by_targets(sl):
    groups = []
    seen = set()
    for order, stmt in enumerate(flat_iteration(sl)):
        targets = set(list_targets(stmt))
        group = [(order, stmt)]
        disjoint = targets.isdisjoint(seen)
        seen |= targets
        if not disjoint:
            groups, old_groups = [], groups
            for old_targets, old_group in old_groups:
                if targets.isdisjoint(old_targets):
                    groups.append((old_targets, old_group))
                else:
                    targets |= old_targets
                    group += old_group
        groups.append((targets, group))
    return [(targets, _resort_statements(stmts))
        for targets, stmts in groups]


def list_special_ios(f, ins, outs, inouts):
    r = set()
    for special in f.specials:
        r |= special.list_ios(ins, outs, inouts)
    return r


class _ClockDomainLister(NodeVisitor):
    def __init__(self):
        self.clock_domains = set()

    def visit_ClockSignal(self, node):
        self.clock_domains.add(node.cd)

    def visit_ResetSignal(self, node):
        self.clock_domains.add(node.cd)

    def visit_clock_domains(self, node):
        for clockname, statements in node.items():
            self.clock_domains.add(clockname)
            self.visit(statements)


def list_clock_domains_expr(f):
    cdl = _ClockDomainLister()
    cdl.visit(f)
    return cdl.clock_domains


def list_clock_domains(f):
    r = list_clock_domains_expr(f)
    for special in f.specials:
        r |= special.list_clock_domains()
    for cd in f.clock_domains:
        r.add(cd.name)
    return r


def is_variable(node):
    if isinstance(node, Signal):
        return node.variable
    elif isinstance(node, _Slice):
        return is_variable(node.value)
    elif isinstance(node, _Part):
        if is_variable(node.offset) != is_variable(node.value):
            raise TypeError
        return is_variable(node.value)
    elif isinstance(node, Cat):
        arevars = list(map(is_variable, node.l))
        r = arevars[0]
        for x in arevars:
            if x != r:
                raise TypeError
        return r
    else:
        raise TypeError


def generate_reset(rst, sl):
    targets = list_targets(sl)
    return [t.eq(t.reset) for t in sorted(targets, key=lambda x: x.duid)
            if not t.reset_less]


def insert_reset(rst, sl):
    return sl + [If(rst, *generate_reset(rst, sl))]


def insert_resets(f):
    newsync = dict()
    for k, v in f.sync.items():
        if f.clock_domains[k].rst is not None:
            newsync[k] = insert_reset(ResetSignal(k), v)
        else:
            newsync[k] = v
    f.sync = newsync


class _Lowerer(NodeTransformer):
    def __init__(self):
        self.target_context = False
        self.extra_stmts = []
        self.comb = []

    def visit_Assign(self, node):
        old_target_context, old_extra_stmts = self.target_context, self.extra_stmts
        self.extra_stmts = []

        self.target_context = True
        lhs = self.visit(node.l)
        self.target_context = False
        rhs = self.visit(node.r)
        r = _Assign(lhs, rhs)
        if self.extra_stmts:
            r = [r] + self.extra_stmts

        self.target_context, self.extra_stmts = old_target_context, old_extra_stmts
        return r


# Basics are FHDL structure elements that back-ends are not required to support
# but can be expressed in terms of other elements (lowered) before conversion.
class _BasicLowerer(_Lowerer):
    def __init__(self, clock_domains):
        self.clock_domains = clock_domains
        _Lowerer.__init__(self)

    def visit_ArrayProxy(self, node):
        # TODO: rewrite without variables
        array_muxed = Signal(value_bits_sign(node), variable=True)
        if self.target_context:
            k = self.visit(node.key)
            cases = {}
            for n, choice in enumerate(node.choices):
                cases[n] = [self.visit_Assign(_Assign(choice, array_muxed))]
            self.extra_stmts.append(Case(k, cases).makedefault())
        else:
            cases = dict((n, _Assign(array_muxed, self.visit(choice)))
                for n, choice in enumerate(node.choices))
            self.comb.append(Case(self.visit(node.key), cases).makedefault())
        return array_muxed

    def visit_ClockSignal(self, node):
        return self.clock_domains[node.cd].clk

    def visit_ResetSignal(self, node):
        rst = self.clock_domains[node.cd].rst
        if rst is None:
            if node.allow_reset_less:
                return 0
            else:
                raise ValueError("Attempted to get reset signal of resetless"
                                 " domain '{}'".format(node.cd))
        else:
            return rst


class _ComplexSliceLowerer(_Lowerer):
    def visit_Slice(self, node):
        if not isinstance(node.value, Signal):
            slice_proxy = Signal(value_bits_sign(node.value))
            if self.target_context:
                a = _Assign(node.value, slice_proxy)
            else:
                a = _Assign(slice_proxy, node.value)
            self.comb.append(self.visit_Assign(a))
            node = _Slice(slice_proxy, node.start, node.stop)
        return NodeTransformer.visit_Slice(self, node)

class _ComplexPartLowerer(_Lowerer):
    def visit_Part(self, node):
        value_proxy = node.value
        offset_proxy = node.offset
        if not isinstance(node.value, Signal):
            value_proxy = Signal(value_bits_sign(node.value))
            if self.target_context:
                a = _Assign(node.value, value_proxy)
            else:
                a = _Assign(value_proxy, node.value)
            self.comb.append(self.visit_Assign(a))
        if not isinstance(node.offset, Signal):
            offset_proxy = Signal(value_bits_sign(node.offset))
            if self.target_context:
                a = _Assign(node.offset, offset_proxy)
            else:
                a = _Assign(offset_proxy, node.offset)
            self.comb.append(self.visit_Assign(a))
        node = _Part(value_proxy, offset_proxy, node.width)
        return NodeTransformer.visit_Part(self, node)

def _apply_lowerer(l, f):
    f = l.visit(f)
    f.comb += l.comb

    for special in sorted(f.specials, key=lambda s: s.duid):
        for obj, attr, direction in special.iter_expressions():
            if direction != SPECIAL_INOUT:
                # inouts are only supported by Migen when connected directly to top-level
                # in this case, they are Signal and never need lowering
                l.comb = []
                l.target_context = direction != SPECIAL_INPUT
                l.extra_stmts = []
                expr = getattr(obj, attr)
                expr = l.visit(expr)
                setattr(obj, attr, expr)
                f.comb += l.comb + l.extra_stmts

    return f


def lower_basics(f):
    return _apply_lowerer(_BasicLowerer(f.clock_domains), f)


def lower_complex_slices(f):
    return _apply_lowerer(_ComplexSliceLowerer(), f)

def lower_complex_parts(f):
    return _apply_lowerer(_ComplexPartLowerer(), f)

class _ClockDomainRenamer(NodeVisitor):
    def __init__(self, old, new):
        self.old = old
        self.new = new

    def visit_ClockSignal(self, node):
        if node.cd == self.old:
            node.cd = self.new

    def visit_ResetSignal(self, node):
        if node.cd == self.old:
            node.cd = self.new


def rename_clock_domain_expr(f, old, new):
    cdr = _ClockDomainRenamer(old, new)
    cdr.visit(f)


def rename_clock_domain(f, old, new):
    rename_clock_domain_expr(f, old, new)
    if new != old:
        if old in f.sync:
            if new in f.sync:
                f.sync[new].extend(f.sync[old])
            else:
                f.sync[new] = f.sync[old]
            del f.sync[old]
    for special in f.specials:
        special.rename_clock_domain(old, new)
    try:
        cd = f.clock_domains[old]
    except KeyError:
        pass
    else:
        cd.rename(new)


def call_special_classmethod(overrides, obj, method, *args, **kwargs):
    cl = obj.__class__
    if cl in overrides:
        cl = overrides[cl]
    if hasattr(cl, method):
        return getattr(cl, method)(obj, *args, **kwargs)
    else:
        return None


def _lower_specials_step(overrides, specials):
    f = _Fragment()
    lowered_specials = set()
    for special in sorted(specials, key=lambda x: x.duid):
        impl = call_special_classmethod(overrides, special, "lower")
        if impl is not None:
            f += impl.get_fragment()
            lowered_specials.add(special)
    return f, lowered_specials


def lower_specials(overrides, f):
    lowered_specials = set()
    while True:
        fs, lowered_specials_step = _lower_specials_step(overrides, f.specials)
        f += fs
        if lowered_specials_step:
            lowered_specials |= lowered_specials_step
            f.specials -= lowered_specials_step
        else:
            break
    return f, lowered_specials



import os
import unittest
import importlib
import pkgutil
import tempfile

from migen import *
from migen.genlib.cdc import MultiReg
from migen.build.lattice import diamond, icestorm, trellis
from migen.build.altera import quartus
from migen.build.xilinx import ise, vivado
import migen.build.platforms


def _toolchain_var(plat):
    if isinstance(plat.toolchain, diamond.LatticeDiamondToolchain):
        return "MIGEN_HAS_DIAMOND"
    elif isinstance(plat.toolchain, icestorm.LatticeIceStormToolchain):
        return "MIGEN_HAS_ICESTORM"
    elif isinstance(plat.toolchain, trellis.LatticeTrellisToolchain):
        return "MIGEN_HAS_TRELLIS"
    elif isinstance(plat.toolchain, quartus.AlteraQuartusToolchain):
        return "MIGEN_HAS_QUARTUS"
    elif isinstance(plat.toolchain, ise.XilinxISEToolchain):
        return "MIGEN_HAS_ISE"
    elif isinstance(plat.toolchain, vivado.XilinxVivadoToolchain):
        return "MIGEN_HAS_VIVADO"
    else:
        raise ValueError("Unrecognized toolchain {} for {}"
                         .format(type(plat.toolchain), type(plat)))


def _find_platforms(mod_root):
    def mk_name(mod, child):
        return ".".join([mod.__name__, child])

    imports = []
    for _, name, is_mod in pkgutil.walk_packages(mod_root.__path__):
        if is_mod:
            new_root = importlib.import_module(mk_name(mod_root, name))
            imports.extend(_find_platforms(new_root))
        else:
            imports.append((mk_name(mod_root, name), name))
    return imports


class TestModulePlatform(Module):
    def __init__(self, plat):
        # FIXME: Somehow incorporate plat.request() into this.
        inp = Signal()
        out = Signal()

        self.specials += MultiReg(inp, out)


class TestExamplesPlatform(unittest.TestCase):
    def test_platforms(self):
        for mod, name in _find_platforms(migen.build.platforms):
            with self.subTest(mod=mod, name=name):
                run_toolchain_var = "MIGEN_RUN_TOOLCHAIN_{}".format(
                                    name.upper())

                plat = importlib.import_module(mod).Platform()
                has_toolchain_var = _toolchain_var(plat)
                if not os.getenv(has_toolchain_var, False):
                    raise unittest.SkipTest("{} not set for {}."
                                            .format(has_toolchain_var, name))

                m = TestModulePlatform(plat)
                with tempfile.TemporaryDirectory(name) as temp_dir:
                    do_build = os.getenv(run_toolchain_var, False)
                    if not do_build:
                        print("{} not set, not running toolchain for {}."
                              .format(run_toolchain_var, name))
                    plat.build(m, build_name=name, build_dir=temp_dir,
                               run=do_build)
